@@ -27,6 +27,8 @@
 #include "AnnotationSpatialModification.h"
 #include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "EventAnnotationTextSubstitutionGet.h"
+#include "EventManager.h"
 #include "MathFunctions.h"
 #include "SceneClass.h"
 #include "SceneClassAssistant.h"
@@ -51,7 +53,7 @@ using namespace caret;
  */
 AnnotationText::AnnotationText(const AnnotationAttributesDefaultTypeEnum::Enum attributeDefaultType,
                                const AnnotationTextFontSizeTypeEnum::Enum fontSizeType)
-: AnnotationTwoDimensionalShape(AnnotationTypeEnum::TEXT,
+: AnnotationOneCoordinateShape(AnnotationTypeEnum::TEXT,
                                 attributeDefaultType),
 AnnotationFontAttributesInterface(),
 m_fontSizeType(fontSizeType)
@@ -72,7 +74,7 @@ m_fontSizeType(fontSizeType)
 AnnotationText::AnnotationText(const AnnotationTypeEnum::Enum type,
                                const AnnotationAttributesDefaultTypeEnum::Enum attributeDefaultType,
                                const AnnotationTextFontSizeTypeEnum::Enum fontSizeType)
-: AnnotationTwoDimensionalShape(type,
+: AnnotationOneCoordinateShape(type,
                                 attributeDefaultType),
 AnnotationFontAttributesInterface(),
 m_fontSizeType(fontSizeType)
@@ -93,7 +95,7 @@ AnnotationText::~AnnotationText()
  *    Object that is copied.
  */
 AnnotationText::AnnotationText(const AnnotationText& obj)
-: AnnotationTwoDimensionalShape(obj),
+: AnnotationOneCoordinateShape(obj),
 AnnotationFontAttributesInterface(),
 m_fontSizeType(obj.m_fontSizeType)
 {
@@ -162,6 +164,7 @@ AnnotationText::initializeAnnotationTextMembers()
     }
     
     m_text = "";
+    m_textWithSubstitutions = "";
     
     /*
      * Assists with attributes that may be saved to scene (controlled by annotation property).
@@ -279,6 +282,95 @@ AnnotationText::getText() const
     return m_text;
 }
 
+/**
+ * Invalidate text substitutions.  This method is
+ * implemented as a virtual method to avoid
+ * dyamic casts since they are slow.
+ */
+void
+AnnotationText::invalidateTextSubstitution()
+{
+    m_textWithSubstitutions.clear();
+}
+
+/**
+ * @return Text with any substitutions applied to the text.
+ */
+AString
+AnnotationText::getTextWithSubstitutionsApplied() const
+{
+    if (m_textWithSubstitutions.isEmpty()) {
+        if ( ! m_text.isEmpty()) {
+            std::vector<int32_t> indices;
+            const QChar substituteChar('$');
+            int32_t index = m_text.indexOf(substituteChar);
+            while (index >= 0) {
+                indices.push_back(index);
+                index = m_text.indexOf(substituteChar, index + 1);
+            }
+            
+            if (indices.size() < 2) {
+                if (indices.size() == 1) {
+                    CaretLogWarning("Text annotation \""
+                                    + m_text
+                                    + "\" is missing substitution delimeters");
+                }
+                
+                m_textWithSubstitutions = m_text;
+            }
+            else {
+                int32_t lastPos = 0;
+                const int32_t numSubsitutions = static_cast<int32_t>(indices.size() / 2);
+                for (int32_t i = 0; i < numSubsitutions; i++) {
+                    const int32_t i2 = i * 2;
+                    CaretAssertVectorIndex(indices, i2+1);
+                    const int32_t indexOne = indices[i2];
+                    const int32_t indexTwo = indices[i2+1];
+                    if (indexTwo > (indexOne + 1)) {
+                        const int32_t nameLen  = indexTwo - indexOne - 1;
+                        AString name = m_text.mid(indexOne + 1, nameLen);
+                        
+                        EventAnnotationTextSubstitutionGet subEvent;
+                        subEvent.addSubstitutionName(name);
+                        EventManager::get()->sendEvent(subEvent.getPointer());
+                        const AString subValue = subEvent.getSubstitutionValueForName(name);
+                        
+                        if (subValue.isEmpty()) {
+                            CaretLogWarning("Unable to find substitution value for name \""
+                                            + name
+                                            + "\"");
+                        }
+                        
+                        const AString txt = m_text.mid(lastPos, indexOne - lastPos);
+                        m_textWithSubstitutions.append(txt);
+                        if (subValue.isEmpty()) {
+                            m_textWithSubstitutions.append("$" + name + "$");
+                        }
+                        else {
+                            m_textWithSubstitutions.append(subValue);
+                        }
+                    }
+                    else {
+                        const AString txt = m_text.mid(lastPos, indexOne - lastPos);
+                        m_textWithSubstitutions.append(txt + "$$");
+                        CaretLogWarning("Text annotation \""
+                                        + m_text
+                                        + "\" contains empty text substitution delimeters ("
+                                        + substituteChar
+                                        + ")");
+                    }
+                    lastPos = indexTwo + 1;
+                }
+                const AString lastTxt = m_text.mid(lastPos);
+                m_textWithSubstitutions.append(lastTxt);
+            }
+            
+        }
+    }
+    
+    return m_textWithSubstitutions;
+}
+
 /**
  * Set the text for an annotation.
  * @param text
@@ -289,6 +381,7 @@ AnnotationText::setText(const AString& text)
 {
     if (text != m_text) {
         m_text = text;
+        m_textWithSubstitutions.clear();
         textAnnotationResetName();
         setModified();
     }
@@ -792,6 +885,7 @@ void
 AnnotationText::copyHelperAnnotationText(const AnnotationText& obj)
 {
     m_text                = obj.m_text;
+    m_textWithSubstitutions.clear();
     m_alignmentHorizontal = obj.m_alignmentHorizontal;
     m_alignmentVertical   = obj.m_alignmentVertical;
     m_font                = obj.m_font;
@@ -820,7 +914,7 @@ AnnotationText::copyHelperAnnotationText(const AnnotationText& obj)
 void
 AnnotationText::applyCoordinatesSizeAndRotationFromOther(const Annotation* otherAnnotation)
 {
-    AnnotationTwoDimensionalShape::applyCoordinatesSizeAndRotationFromOther(otherAnnotation);
+    AnnotationOneCoordinateShape::applyCoordinatesSizeAndRotationFromOther(otherAnnotation);
     
     /*
      * Text size may change when an annotation is moved to a different
@@ -940,11 +1034,13 @@ AnnotationText::applySpatialModification(const AnnotationSpatialModification& sp
         case AnnotationSizingHandleTypeEnum::ANNOTATION_SIZING_HANDLE_ROTATION:
             operationSupportedFlag = true;
             break;
+        case AnnotationSizingHandleTypeEnum::ANNOTATION_SIZING_HANDLE_POLY_LINE_COORDINATE:
+            break;
     }
     
     bool validFlag = false;
     if (operationSupportedFlag) {
-        validFlag = AnnotationTwoDimensionalShape::applySpatialModification(spatialModification);
+        validFlag = AnnotationOneCoordinateShape::applySpatialModification(spatialModification);
     }
     
     return validFlag;
@@ -966,7 +1062,7 @@ void
 AnnotationText::saveSubClassDataToScene(const SceneAttributes* sceneAttributes,
                                             SceneClass* sceneClass)
 {
-    AnnotationTwoDimensionalShape::saveSubClassDataToScene(sceneAttributes,
+    AnnotationOneCoordinateShape::saveSubClassDataToScene(sceneAttributes,
                                                            sceneClass);
     if (testProperty(Property::SCENE_CONTAINS_ATTRIBUTES)) {
         sceneClass->addBoolean("hasAttributesFlag", true);
@@ -991,7 +1087,7 @@ void
 AnnotationText::restoreSubClassDataFromScene(const SceneAttributes* sceneAttributes,
                                                  const SceneClass* sceneClass)
 {
-    AnnotationTwoDimensionalShape::restoreSubClassDataFromScene(sceneAttributes,
+    AnnotationOneCoordinateShape::restoreSubClassDataFromScene(sceneAttributes,
                                                                 sceneClass);
     if (testProperty(Property::SCENE_CONTAINS_ATTRIBUTES)) {
         /*
