@@ -24,6 +24,7 @@
 #undef __DISPLAY_PROPERTIES_FOCI_DECLARE__
 
 #include "CaretAssert.h"
+#include "DisplayPropertyDataFloat.h"
 #include "SceneAttributes.h"
 #include "SceneClass.h"
 #include "SceneClassAssistant.h"
@@ -44,29 +45,15 @@ DisplayPropertiesFoci::DisplayPropertiesFoci()
 : DisplayProperties()
 {
     const CaretColorEnum::Enum defaultColor = CaretColorEnum::BLACK;
+    
+    m_fociSizePercentage.reset(new DisplayPropertyDataFloat(4.0));
+    m_fociSymbolSizeType.initialize(IdentificationSymbolSizeTypeEnum::MILLIMETERS);
+    resetPrivate();
+    
+    m_sceneAssistant->add("m_fociSizePercentage", "DisplayPropertyDataFloat", m_fociSizePercentage.get());
+    m_sceneAssistant->add("m_fociSymbolSizeType", "DisplayPropertyDataEnum", &m_fociSymbolSizeType);
 
-    for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS; i++) {
-        m_displayGroup[i] = DisplayGroupEnum::getDefaultValue();
-        m_pasteOntoSurfaceInTab[i] = false;
-        m_displayStatusInTab[i] = false;
-        m_contralateralDisplayStatusInTab[i] = false;
-        m_fociSizeInTab[i] = 4.0;
-        m_coloringTypeInTab[i] = FeatureColoringTypeEnum::FEATURE_COLORING_TYPE_NAME;
-        m_drawingTypeInTab[i] = FociDrawingTypeEnum::DRAW_AS_SQUARES;
-        m_standardColorTypeInTab[i] = defaultColor;
-    }
-    
-    for (int32_t i = 0; i < DisplayGroupEnum::NUMBER_OF_GROUPS; i++) {
-        m_pasteOntoSurfaceInDisplayGroup[i] = false;
-        m_displayStatusInDisplayGroup[i] = false;
-        m_contralateralDisplayStatusInDisplayGroup[i] = false;
-        m_fociSizeInDisplayGroup[i] = 4.0;
-        m_coloringTypeInDisplayGroup[i] = FeatureColoringTypeEnum::FEATURE_COLORING_TYPE_NAME;
-        m_drawingTypeInDisplayGroup[i] = FociDrawingTypeEnum::DRAW_AS_SQUARES;
-        m_standardColorTypeInDisplayGroup[i] = defaultColor;
-    }
-    
-    m_sceneAssistant->addTabIndexedEnumeratedTypeArray<DisplayGroupEnum,DisplayGroupEnum::Enum>("m_displayGroup", 
+    m_sceneAssistant->addTabIndexedEnumeratedTypeArray<DisplayGroupEnum,DisplayGroupEnum::Enum>("m_displayGroup",
                                                                                                 m_displayGroup);
     m_sceneAssistant->addTabIndexedBooleanArray("m_pasteOntoSurfaceInTab", 
                                m_pasteOntoSurfaceInTab);
@@ -80,6 +67,8 @@ DisplayPropertiesFoci::DisplayPropertiesFoci()
                                                                                                         m_coloringTypeInTab);
     m_sceneAssistant->addTabIndexedEnumeratedTypeArray<FociDrawingTypeEnum,FociDrawingTypeEnum::Enum>("m_drawingTypeInTab", 
                                                                                                       m_drawingTypeInTab);
+    m_sceneAssistant->addTabIndexedEnumeratedTypeArray<FociDrawingProjectionTypeEnum,FociDrawingProjectionTypeEnum::Enum>("m_drawingProjectionTypeInTab",
+                                                                                                                          m_drawingProjectionTypeInTab);
     
     m_sceneAssistant->addArray("m_pasteOntoSurfaceInDisplayGroup", 
                                m_pasteOntoSurfaceInDisplayGroup,
@@ -106,6 +95,10 @@ DisplayPropertiesFoci::DisplayPropertiesFoci()
                                                                                m_drawingTypeInDisplayGroup,
                                                                                DisplayGroupEnum::NUMBER_OF_GROUPS,
                                                                                FociDrawingTypeEnum::DRAW_AS_SQUARES);
+    m_sceneAssistant->addArray<FociDrawingProjectionTypeEnum,FociDrawingProjectionTypeEnum::Enum>("m_drawingProjectionTypeInDisplayGroup",
+                                                                                                  m_drawingProjectionTypeInDisplayGroup,
+                                                                                                  DisplayGroupEnum::NUMBER_OF_GROUPS,
+                                                                                                  FociDrawingProjectionTypeEnum::PROJECTED);
     m_sceneAssistant->addArray<CaretColorEnum, CaretColorEnum::Enum>("m_standardColorTypeInDisplayGroup",
                                                                      m_standardColorTypeInDisplayGroup,
                                                                      DisplayGroupEnum::NUMBER_OF_GROUPS,
@@ -141,6 +134,7 @@ DisplayPropertiesFoci::copyDisplayProperties(const int32_t sourceTabIndex,
     m_contralateralDisplayStatusInTab[targetTabIndex] = m_contralateralDisplayStatusInTab[sourceTabIndex];
     m_displayStatusInTab[targetTabIndex]    = m_displayStatusInTab[sourceTabIndex];
     m_drawingTypeInTab[targetTabIndex]      = m_drawingTypeInTab[sourceTabIndex];
+    m_drawingProjectionTypeInTab[targetTabIndex] = m_drawingProjectionTypeInTab[sourceTabIndex];
     m_fociSizeInTab[targetTabIndex]         = m_fociSizeInTab[sourceTabIndex];
     m_pasteOntoSurfaceInTab[targetTabIndex] = m_pasteOntoSurfaceInTab[sourceTabIndex];
     m_standardColorTypeInTab[targetTabIndex] = m_standardColorTypeInTab[sourceTabIndex];
@@ -153,12 +147,39 @@ DisplayPropertiesFoci::copyDisplayProperties(const int32_t sourceTabIndex,
 void 
 DisplayPropertiesFoci::reset()
 {
-//    for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS; i++) {
-//        m_displayStatus[i] = true;
-//        m_contralateralDisplayStatus[i] = false;
-//        m_displayGroup[i] = DisplayGroupEnum::DISPLAY_ALL_WINDOWS;
-//        m_pasteOntoSurface[i] = false;
-//    }
+    resetPrivate();
+}
+
+void
+DisplayPropertiesFoci::resetPrivate()
+{
+    const CaretColorEnum::Enum defaultColor = CaretColorEnum::BLACK;
+    
+    for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS; i++) {
+        m_displayGroup[i] = DisplayGroupEnum::getDefaultValue();
+        m_pasteOntoSurfaceInTab[i] = false;
+        m_displayStatusInTab[i] = false;
+        m_contralateralDisplayStatusInTab[i] = false;
+        m_fociSizeInTab[i] = 4.0;
+        m_coloringTypeInTab[i] = FeatureColoringTypeEnum::FEATURE_COLORING_TYPE_NAME;
+        m_drawingTypeInTab[i] = FociDrawingTypeEnum::DRAW_AS_SQUARES;
+        m_drawingProjectionTypeInTab[i] = FociDrawingProjectionTypeEnum::PROJECTED;
+        m_standardColorTypeInTab[i] = defaultColor;
+    }
+    
+    for (int32_t i = 0; i < DisplayGroupEnum::NUMBER_OF_GROUPS; i++) {
+        m_pasteOntoSurfaceInDisplayGroup[i] = false;
+        m_displayStatusInDisplayGroup[i] = false;
+        m_contralateralDisplayStatusInDisplayGroup[i] = false;
+        m_fociSizeInDisplayGroup[i] = 4.0;
+        m_coloringTypeInDisplayGroup[i] = FeatureColoringTypeEnum::FEATURE_COLORING_TYPE_NAME;
+        m_drawingTypeInDisplayGroup[i] = FociDrawingTypeEnum::DRAW_AS_SQUARES;
+        m_drawingProjectionTypeInDisplayGroup[i] = FociDrawingProjectionTypeEnum::PROJECTED;
+        m_standardColorTypeInDisplayGroup[i] = defaultColor;
+    }
+    
+    m_fociSizePercentage->setAllValues(4.0);
+    m_fociSymbolSizeType.setAllValues(IdentificationSymbolSizeTypeEnum::MILLIMETERS);
 }
 
 /**
@@ -297,13 +318,49 @@ DisplayPropertiesFoci::setDisplayGroupForTab(const int32_t browserTabIndex,
     m_displayGroup[browserTabIndex] = displayGroup;
 }
 
+/**
+ * @return  Size type for foci symbol
+ * @param displayGroup
+ *     Display group.
+ * @param tabIndex
+ *    Index of browser tab.
+ */
+IdentificationSymbolSizeTypeEnum::Enum
+DisplayPropertiesFoci::getFociSymbolSizeType(const DisplayGroupEnum::Enum displayGroup,
+                                             const int32_t tabIndex) const
+{
+    return m_fociSymbolSizeType.getValue(displayGroup,
+                                         tabIndex);
+}
+
+/**
+ * Set the foci size to the given value.
+ * @param displayGroup
+ *     Display group.
+ * @param tabIndex
+ *    Index of browser tab.
+ * @param sizeType
+ *     Size type for foci symbol.
+ */
+void
+DisplayPropertiesFoci::setFociSymbolSizeType(const DisplayGroupEnum::Enum displayGroup,
+                                             const int32_t tabIndex,
+                                             const IdentificationSymbolSizeTypeEnum::Enum sizeType)
+{
+    m_fociSymbolSizeType.setValue(displayGroup,
+                                  tabIndex,
+                                  sizeType);
+}
+
 /**
  * @return The foci size.
  * @param displayGroup
  *     Display group.
+ * @param tabIndex
+ *     Index of the tab
  */
 float 
-DisplayPropertiesFoci::getFociSize(const DisplayGroupEnum::Enum displayGroup,
+DisplayPropertiesFoci::getFociSizeMillimeters(const DisplayGroupEnum::Enum displayGroup,
                                    const int32_t tabIndex) const
 {
     CaretAssertArrayIndex(m_fociSizeInDisplayGroup, 
@@ -322,11 +379,13 @@ DisplayPropertiesFoci::getFociSize(const DisplayGroupEnum::Enum displayGroup,
  * Set the foci size to the given value.
  * @param displayGroup
  *     Display group.
+ * @param tabIndex
+ *     Index of the tab
  * @param fociSize
  *     New value for foci size.
  */
 void 
-DisplayPropertiesFoci::setFociSize(const DisplayGroupEnum::Enum displayGroup,
+DisplayPropertiesFoci::setFociSizeMillimeters(const DisplayGroupEnum::Enum displayGroup,
                                    const int32_t tabIndex,
                                    const float fociSize)
 {
@@ -344,6 +403,40 @@ DisplayPropertiesFoci::setFociSize(const DisplayGroupEnum::Enum displayGroup,
     }
 }
 
+/**
+ * @return The foci size.
+ * @param displayGroup
+ *     Display group.
+ * @param tabIndex
+ *     Index of the tab
+ */
+float
+DisplayPropertiesFoci::getFociSizePercentage(const DisplayGroupEnum::Enum displayGroup,
+                            const int32_t tabIndex) const
+{
+    return m_fociSizePercentage->getValue(displayGroup,
+                                          tabIndex);
+}
+
+/**
+ * Set the foci size to the given value.
+ * @param displayGroup
+ *     Display group.
+ * @param tabIndex
+ *     Index of the tab
+ * @param fociSize
+ *     New value for foci size.
+ */
+void
+DisplayPropertiesFoci::setFociSizePercentage(const DisplayGroupEnum::Enum displayGroup,
+                           const int32_t tabIndex,
+                           const float pointSize)
+{
+    m_fociSizePercentage->setValue(displayGroup,
+                                   tabIndex,
+                                   pointSize);
+}
+
 /**
  * @return The coloring type.
  * @param displayGroup
@@ -485,6 +578,53 @@ DisplayPropertiesFoci::setDrawingType(const DisplayGroupEnum::Enum displayGroup,
     }
 }
 
+/**
+ * @param displayGroup
+ *     Display group.
+ * @return The drawing type.
+ */
+FociDrawingProjectionTypeEnum::Enum
+DisplayPropertiesFoci::getDrawingProjectionType(const DisplayGroupEnum::Enum displayGroup,
+                                                const int32_t tabIndex) const
+{
+    CaretAssertArrayIndex(m_drawingProjectionTypeInDisplayGroup,
+                          DisplayGroupEnum::NUMBER_OF_GROUPS,
+                          static_cast<int32_t>(displayGroup));
+    if (displayGroup == DisplayGroupEnum::DISPLAY_GROUP_TAB) {
+        CaretAssertArrayIndex(m_drawingProjectionTypeInTab,
+                              BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                              tabIndex);
+        return m_drawingProjectionTypeInTab[tabIndex];
+    }
+    return m_drawingProjectionTypeInDisplayGroup[displayGroup];
+}
+
+/**
+ * Set the drawing type to the given value.
+ * @param displayGroup
+ *     Display group.
+ * @param drawingType
+ *     New value for drawing type.
+ */
+void
+DisplayPropertiesFoci::setDrawingProjectionType(const DisplayGroupEnum::Enum displayGroup,
+                                      const int32_t tabIndex,
+                                      const FociDrawingProjectionTypeEnum::Enum drawingType)
+{
+    CaretAssertArrayIndex(m_drawingProjectionTypeInDisplayGroup,
+                          DisplayGroupEnum::NUMBER_OF_GROUPS,
+                          static_cast<int32_t>(displayGroup));
+    if (displayGroup == DisplayGroupEnum::DISPLAY_GROUP_TAB) {
+        CaretAssertArrayIndex(m_drawingProjectionTypeInTab,
+                              BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                              tabIndex);
+        m_drawingProjectionTypeInTab[tabIndex] = drawingType;
+    }
+    else {
+        m_drawingProjectionTypeInDisplayGroup[displayGroup] = drawingType;
+    }
+}
+
 /**
  * Set paste onto surface so the foci are placed directly on the surface.
  * @param displayGroup
@@ -586,6 +726,7 @@ DisplayPropertiesFoci::restoreFromScene(const SceneAttributes* sceneAttributes,
     if (sceneClass == NULL) {
         return;
     }
+    resetPrivate();
     
     m_sceneAssistant->restoreMembers(sceneAttributes, 
                                      sceneClass);
@@ -596,6 +737,5 @@ DisplayPropertiesFoci::restoreFromScene(const SceneAttributes* sceneAttributes,
         case SceneTypeEnum::SCENE_TYPE_GENERIC:
             break;
     }
-    
 }
 
