@@ -57,6 +57,7 @@
 #include "AlgorithmCiftiReorder.h"
 #include "AlgorithmCiftiReplaceStructure.h"
 #include "AlgorithmCiftiResample.h"
+#include "AlgorithmCiftiRestrictDenseMap.h"
 #include "AlgorithmCiftiROIsFromExtrema.h"
 #include "AlgorithmCiftiSeparate.h"
 #include "AlgorithmCiftiSmoothing.h"
@@ -133,6 +134,7 @@
 #include "AlgorithmVolumeParcelSmoothing.h"
 #include "AlgorithmVolumeReduce.h"
 #include "AlgorithmVolumeRemoveIslands.h"
+#include "AlgorithmVolumeResample.h"
 #include "AlgorithmVolumeROIsFromExtrema.h"
 #include "AlgorithmVolumeSmoothing.h"
 #include "AlgorithmVolumeTFCE.h"
@@ -164,7 +166,6 @@
 #include "OperationCiftiMerge.h"
 #include "OperationCiftiPalette.h"
 #include "OperationCiftiResampleDconnMemory.h"
-#include "AlgorithmCiftiRestrictDenseMap.h"
 #include "OperationCiftiROIAverage.h"
 #include "OperationCiftiSeparateAll.h"
 #include "OperationCiftiStats.h"
@@ -248,7 +249,7 @@
 
 #include "CaretLogger.h"
 #include "dot_wrapper.h"
-#include "StructureEnum.h"
+#include "CaretCommandGlobalOptions.h"
 
 #include <iostream>
 #include <map>
@@ -380,7 +381,6 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmSurfaceSphereProjectUnproject()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmSurfaceToSurface3dDistance()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmSurfaceWedgeVolume()));
-    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeAffineResample()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeAllLabelsToROIs()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeDilate()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeDistortion()));
@@ -399,13 +399,13 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeParcelSmoothing()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeReduce()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeRemoveIslands()));
+    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeResample()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeROIsFromExtrema()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeSmoothing()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeTFCE()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeToSurfaceMapping()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeVectorOperation()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeWarpfieldAffineRegression()));
-    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeWarpfieldResample()));
     
     this->commandOperations.push_back(new CommandParser(new AutoOperationAddToSpecFile()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBackendAverageDenseROI()));
@@ -509,6 +509,8 @@ CommandOperationManager::CommandOperationManager()
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationCiftiSeparateAll()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationMetricVertexSum()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationSetMapName()));
+    this->deprecatedOperations.push_back(new CommandParser(new AutoAlgorithmVolumeAffineResample()));
+    this->deprecatedOperations.push_back(new CommandParser(new AutoAlgorithmVolumeWarpfieldResample()));
 }
 
 /**
@@ -550,7 +552,7 @@ namespace
         map<AString, int16_t>::iterator iter = nameToCode.find(input);
         if (iter == nameToCode.end())
         {
-            throw CommandException("Unrecognized cifti datatype: '" + input + "'");
+            throw CommandException("Unrecognized nifti datatype: '" + input + "'");
         }
         return iter->second;
     }
@@ -587,9 +589,9 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
             CaretLogWarning("SIMD type '" + DotSIMDEnum::toName(impl) + "' not supported (could be cpu, compiler, or build options), using '" + DotSIMDEnum::toName(retval) + "'");
         }
     }
-    int16_t ciftiDType = NIFTI_TYPE_FLOAT32;
-    bool ciftiScale = false;
-    double ciftiMin = -1.0, ciftiMax = -1.0;
+    int16_t ciftiDType = NIFTI_TYPE_FLOAT32, niftiDType = NIFTI_TYPE_FLOAT32;
+    bool ciftiScale = false, niftiScale = false;
+    double ciftiMin = -1.0, ciftiMax = -1.0, niftiMin = -1.0, niftiMax = -1.0;
     if (getGlobalOption(parameters, "-cifti-output-datatype", 1, globalOptionArgs))
     {
         ciftiDType = stringToCiftiType(globalOptionArgs[0]);
@@ -603,6 +605,23 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
         ciftiMax = globalOptionArgs[1].toDouble(&valid);
         if (!valid) throw CommandException("non-numeric option to -cifti-output-range: '" + globalOptionArgs[1] + "'");
     }
+    if (getGlobalOption(parameters, "-nifti-output-datatype", 1, globalOptionArgs))
+    {
+        niftiDType = stringToCiftiType(globalOptionArgs[0]);
+    }
+    if (getGlobalOption(parameters, "-nifti-output-range", 2, globalOptionArgs))
+    {
+        niftiScale = true;
+        bool valid = false;
+        niftiMin = globalOptionArgs[0].toDouble(&valid);
+        if (!valid) throw CommandException("non-numeric option to -nifti-output-range: '" + globalOptionArgs[0] + "'");
+        niftiMax = globalOptionArgs[1].toDouble(&valid);
+        if (!valid) throw CommandException("non-numeric option to -nifti-output-range: '" + globalOptionArgs[1] + "'");
+    }
+    if (getGlobalOption(parameters, "-cifti-read-memory", 0, globalOptionArgs))
+    {
+        caret_global_command_options.m_ciftiReadMemory = true;
+    }
 
     const uint64_t numberOfCommands = this->commandOperations.size();
     const uint64_t numberOfDeprecated = this->deprecatedOperations.size();
@@ -676,6 +695,14 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
             {
                 cout << operation->getHelpInformation(myProgramName) << endl;
             } else {
+                if (niftiScale)
+                {
+                    operation->setCiftiOutputDTypeAndScale(niftiDType, niftiMin, niftiMax);
+                    operation->setVolumeOutputDTypeAndScale(niftiDType, niftiMin, niftiMax);
+                } else {
+                    operation->setCiftiOutputDTypeNoScale(niftiDType);
+                    operation->setVolumeOutputDTypeNoScale(niftiDType);
+                }
                 if (ciftiScale)
                 {
                     operation->setCiftiOutputDTypeAndScale(ciftiDType, ciftiMin, ciftiMax);
@@ -721,14 +748,25 @@ AString CommandOperationManager::doCompletion(ProgramParameters& parameters, con
     OptionInfo ciftiDTypeInfo = parseGlobalOption(parameters, "-cifti-output-datatype", 1, globalOptionArgs, true);
     if (ciftiDTypeInfo.specified && !ciftiDTypeInfo.complete)
     {
-        return "wordlist INT8 UINT8 INT16 UINT16 INT32 UINT32 INT64 UINT64 FLOAT32 FLOAT64 FLOAT128";
+        return "wordlist INT8\\ UINT8\\ INT16\\ UINT16\\ INT32\\ UINT32\\ INT64\\ UINT64\\ FLOAT32\\ FLOAT64\\ FLOAT128";
     }
     OptionInfo ciftiRangeInfo = parseGlobalOption(parameters, "-cifti-output-range", 2, globalOptionArgs, true);
     if (ciftiRangeInfo.specified && !ciftiRangeInfo.complete)
     {//can't tab complete a literal number
         return "";
     }
-    ret = "wordlist -disable-provenance\\ -logging\\ -simd\\ -cifti-output-datatype\\ -cifti-output-range";//we could prevent suggesting an already-provided global option, but that would be a bit surprising
+    OptionInfo niftiDTypeInfo = parseGlobalOption(parameters, "-nifti-output-datatype", 1, globalOptionArgs, true);
+    if (niftiDTypeInfo.specified && !niftiDTypeInfo.complete)
+    {
+        return "wordlist INT8\\ UINT8\\ INT16\\ UINT16\\ INT32\\ UINT32\\ INT64\\ UINT64\\ FLOAT32\\ FLOAT64\\ FLOAT128";
+    }
+    OptionInfo niftiRangeInfo = parseGlobalOption(parameters, "-nifti-output-range", 2, globalOptionArgs, true);
+    if (niftiRangeInfo.specified && !niftiRangeInfo.complete)
+    {
+        return "";
+    }
+    /*OptionInfo ciftiReadMemInfo = */parseGlobalOption(parameters, "-cifti-read-memory", 0, globalOptionArgs, true);
+    ret = "wordlist -disable-provenance\\ -logging\\ -simd\\ -cifti-output-datatype\\ -cifti-output-range\\ -nifti-output-datatype\\ -nifti-output-range\\ -cifti-read-memory";//we could prevent suggesting an already-provided global option, but that would be a bit surprising
     const uint64_t numberOfCommands = this->commandOperations.size();
     const uint64_t numberOfDeprecated = this->deprecatedOperations.size();
     if (!parameters.hasNext())
@@ -1084,10 +1122,10 @@ void CommandOperationManager::printGlobalOptions()
     cout << "   -disable-provenance               don't generate provenance info in output" << endl;
     cout << "                                        files" << endl;
     cout << endl;
-    cout << "   -cifti-output-datatype <type>     write cifti output with the given" << endl;
-    cout << "                                        datatype (default FLOAT32), note that" << endl;
-    cout << "                                        calculation precision is only float32," << endl;
-    cout << "                                        valid values are:" << endl;
+    cout << "   -nifti-output-datatype <type>     write cifti and volume output with the" << endl;
+    cout << "                                        given datatype (default FLOAT32), note" << endl;
+    cout << "                                        that calculation precision is only" << endl;
+    cout << "                                        float32, valid values are:" << endl;
     cout << "                          INT8" << endl;
     cout << "                          UINT8" << endl;
     cout << "                          INT16" << endl;
@@ -1101,11 +1139,12 @@ void CommandOperationManager::printGlobalOptions()
     cout << "                          FLOAT128" << endl;
     cout << endl;
     //guide for wrap, assuming 80 columns:                                                  |
-    cout << "   -cifti-output-range <min> <max>   write cifti output with scaling and offset" << endl;
-    cout << "                                        header fields such that <min> and <max>" << endl;
-    cout << "                                        are the most extreme values that can be" << endl;
-    cout << "                                        represented, mostly useful with integer" << endl;
-    cout << "                                        output datatypes (see above)" << endl;
+    cout << "   -nifti-output-range <min> <max>   write cifti and volume output with scaling" << endl;
+    cout << "                                        and offset header fields such that" << endl;
+    cout << "                                        <min> and <max> are the most extreme" << endl;
+    cout << "                                        values that can be represented, not" << endl;
+    cout << "                                        recommended with floating point types" << endl;
+    cout << "                                        (see above)" << endl;
     cout << endl;
     cout << "   -logging <level>                  set the logging level, valid values are:" << endl;
     vector<LogLevelEnum::Enum> logLevels;
@@ -1128,6 +1167,13 @@ void CommandOperationManager::printGlobalOptions()
         cout << "         " << DotSIMDEnum::toName(*iter) << endl;
     }
     cout << endl;
+    cout << "   -cifti-read-memory                read cifti input files into memory, to" << endl;
+    cout << "                                        avoid hitting limits on number of open" << endl;
+    cout << "                                        files" << endl;
+    cout << endl;
+    cout << "   -cifti-output-datatype <type>     deprecated, only affects cifti outputs" << endl;
+    cout << "   -cifti-output-range <min> <max>   deprecated, only affects cifti outputs" << endl;
+    cout << endl;
 }
 
 void CommandOperationManager::printCiftiHelp()
