@@ -23,6 +23,13 @@
 #include "DataFileTypeEnum.h"
 #undef __DATA_FILE_TYPE_ENUM_DECLARE__
 
+#include <set>
+
+#include <QDir>
+#include <QImageReader>
+#include <QImageWriter>
+#include <QMovie>
+
 #include "CaretAssert.h"
 #include "CaretLogger.h"
 
@@ -72,32 +79,49 @@ DataFileTypeEnum::DataFileTypeEnum(const Enum enumValue,
     this->oneStructureFlag = fileIsUsedWithOneStructure;
     
     if (fileExtensionOne.isEmpty() == false) {
-        this->fileExtensions.push_back(fileExtensionOne);
+        this->readAndWriteFileExtensions.push_back(fileExtensionOne);
     }
     if (fileExtensionTwo.isEmpty() == false) {
-        this->fileExtensions.push_back(fileExtensionTwo);
+        this->readAndWriteFileExtensions.push_back(fileExtensionTwo);
     }
     if (fileExtensionThree.isEmpty() == false) {
-        this->fileExtensions.push_back(fileExtensionThree);
+        this->readAndWriteFileExtensions.push_back(fileExtensionThree);
     }
     if (fileExtensionFour.isEmpty() == false) {
-        this->fileExtensions.push_back(fileExtensionFour);
+        this->readAndWriteFileExtensions.push_back(fileExtensionFour);
     }
 
-    AString filterText = this->guiName + " Files (";
-    
-    
-    for (std::vector<AString>::const_iterator iter = this->fileExtensions.begin();
-         iter != this->fileExtensions.end();
-         iter++) {
-        if (iter != fileExtensions.begin()) {
-            filterText += " ";
-        }
-        filterText += ("*." + *iter);
+    if (this->enumValue == DataFileTypeEnum::IMAGE) {
+        /*
+         * Images support different extensions for reading and writing
+         */
+        AString defaultWriteImageExtension;
+        getWorkbenchSupportedImageFileExtensions(this->readFileExtensions,
+                                                 this->writeFileExtensions,
+                                                 defaultWriteImageExtension);
+        
+        std::set<AString> uniqueExtensions(this->readFileExtensions.begin(),
+                                           this->readFileExtensions.end());
+        uniqueExtensions.insert(this->writeFileExtensions.begin(),
+                                this->writeFileExtensions.end());
+        
+        this->readAndWriteFileExtensions.clear();
+        this->readAndWriteFileExtensions.insert(this->readAndWriteFileExtensions.end(),
+                                                uniqueExtensions.begin(),
+                                                uniqueExtensions.end());
+    }
+    else {
+        /*
+         * Most files use same extension for reading or writing
+         */
+        this->readFileExtensions  = this->readAndWriteFileExtensions;
+        this->writeFileExtensions = this->readAndWriteFileExtensions;
     }
-    filterText += ")";
     
-    this->qFileDialogNameFilter = filterText;
+    this->qReadFileDialogNameFilter  = createQFileDialogNameFilter(this->guiName,
+                                                                   this->readFileExtensions);
+    this->qWriteFileDialogNameFilter = createQFileDialogNameFilter(this->guiName,
+                                                                   this->writeFileExtensions);
 }
 
 /**
@@ -251,10 +275,7 @@ DataFileTypeEnum::initialize()
                                         "Image",
                                         "IMAGE",
                                         false,
-                                        "jpg",
-                                        "jpeg",
-                                        "png",
-                                        "ppm"));
+                                        "XXX")); /* Extensions for IMAGE set in constructor */
     
     enumData.push_back(DataFileTypeEnum(LABEL,
                                         "LABEL", 
@@ -564,7 +585,8 @@ DataFileTypeEnum::fromQFileDialogFilter(const AString& qFileDialogNameFilter,
          iter != enumData.end();
          iter++) {
         const DataFileTypeEnum& d = *iter;
-        if (d.qFileDialogNameFilter == qFileDialogNameFilter) {
+        if ((d.qReadFileDialogNameFilter == qFileDialogNameFilter)
+            && (d.qWriteFileDialogNameFilter == qFileDialogNameFilter)) {
             enumValue = d.enumValue;
             validFlag = true;
             break;
@@ -581,7 +603,7 @@ DataFileTypeEnum::fromQFileDialogFilter(const AString& qFileDialogNameFilter,
 }
 
 /**
- * Get the file filter text for use in a QFileDialog.
+ * Get the file filter text for use in a QFileDialog for READING files
  *
  * @param enumValue
  *     Enumerated type for file filter.
@@ -589,11 +611,27 @@ DataFileTypeEnum::fromQFileDialogFilter(const AString& qFileDialogNameFilter,
  *     Text containing file filter.
  */
 AString 
-DataFileTypeEnum::toQFileDialogFilter(const Enum enumValue)
+DataFileTypeEnum::toQFileDialogFilterForReading(const Enum enumValue)
 {
     if (initializedFlag == false) initialize();
     const DataFileTypeEnum* enumInstance = findData(enumValue);
-    return enumInstance->qFileDialogNameFilter;
+    return enumInstance->qReadFileDialogNameFilter;
+}
+
+/**
+ * Get the file filter text for use in a QFileDialog for WRITING files
+ *
+ * @param enumValue
+ *     Enumerated type for file filter.
+ * @return
+ *     Text containing file filter.
+ */
+AString
+DataFileTypeEnum::toQFileDialogFilterForWriting(const Enum enumValue)
+{
+    if (initializedFlag == false) initialize();
+    const DataFileTypeEnum* enumInstance = findData(enumValue);
+    return enumInstance->qWriteFileDialogNameFilter;
 }
 
 /**
@@ -610,6 +648,60 @@ DataFileTypeEnum::isFileUsedWithOneStructure(const Enum enumValue)
     return enumInstance->oneStructureFlag;
 }
 
+/**
+ * @return All wild card matching (*.ext) for the given enum value.
+ * @param enumValue
+ *     Enumerated type for file extensions.
+ */
+std::vector<AString>
+DataFileTypeEnum::getWildCardMatching(const Enum enumValue)
+{
+    if (initializedFlag == false) initialize();
+    
+    const DataFileTypeEnum* enumInstance = findData(enumValue);
+    std::vector<AString> wildcards;
+    for (auto ext : enumInstance->readAndWriteFileExtensions) {
+        wildcards.push_back("*." + ext);
+    }
+    
+    return wildcards;
+}
+
+/**
+ * Get files for the given enumerated type in the given directory
+ * @param enumValue
+ *    Enumerated type for file extensions.
+ * @param directoryPath
+ *    Name of directory
+ * @return Files in the directory for the type
+ */
+std::vector<AString>
+DataFileTypeEnum::getFilesInDirectory(const Enum enumValue,
+                                      const AString& directoryPath)
+{
+    std::vector<AString> wildCards  = DataFileTypeEnum::getWildCardMatching(enumValue);
+    
+    QStringList fileNameFilters;
+    for (auto w : wildCards) {
+        fileNameFilters.append(w);
+    }
+    
+    QDir::Filters typeFilter(QDir::Files);
+    
+    QDir dir(directoryPath);
+    std::vector<AString> filenamesOut;
+
+    QFileInfoList fileInfoList = dir.entryInfoList(fileNameFilters,
+                                                   typeFilter);
+    QListIterator<QFileInfo> iter(fileInfoList);
+    while (iter.hasNext()) {
+        filenamesOut.push_back(iter.next().absoluteFilePath());
+    }
+
+    return filenamesOut;
+}
+
+
 /**
  * @return All valid file extensions for the given enum value.
  * @param enumValue
@@ -620,7 +712,7 @@ DataFileTypeEnum::getAllFileExtensions(const Enum enumValue)
 {
     if (initializedFlag == false) initialize();
     const DataFileTypeEnum* enumInstance = findData(enumValue);
-    return enumInstance->fileExtensions;
+    return enumInstance->readAndWriteFileExtensions;
 }
 
 /**
@@ -709,8 +801,8 @@ DataFileTypeEnum::getFilesExtensionsForEveryFile(const bool includeNonWritableFi
         
         if (validFlag) {
             allExtensions.insert(allExtensions.end(),
-                                 enumIter->fileExtensions.begin(),
-                                 enumIter->fileExtensions.end());
+                                 enumIter->readAndWriteFileExtensions.begin(),
+                                 enumIter->readAndWriteFileExtensions.end());
         }
     }
     
@@ -742,8 +834,8 @@ DataFileTypeEnum::addFileExtensionIfMissing(const AString& filenameIn,
      * See if filename ends with any of the available extensions for the
      * given data file type.
      */
-    for (std::vector<AString>::const_iterator iter = enumInstance->fileExtensions.begin();
-         iter != enumInstance->fileExtensions.end();
+    for (std::vector<AString>::const_iterator iter = enumInstance->writeFileExtensions.begin();
+         iter != enumInstance->writeFileExtensions.end();
          iter++) {
         const AString ext = ("." + *iter);
         if (filename.endsWith(ext)) {
@@ -774,9 +866,22 @@ DataFileTypeEnum::toFileExtension(const Enum enumValue)
     const DataFileTypeEnum* enumInstance = findData(enumValue);
     
     AString ext = "file";
-    if (enumInstance->fileExtensions.empty() == false) {
-        ext = enumInstance->fileExtensions[0];
+    
+    if ( ! enumInstance->readAndWriteFileExtensions.empty()) {
+        ext = enumInstance->readAndWriteFileExtensions[0];
     }
+    
+    if (enumValue == IMAGE) {
+        /*
+         * Image file's preferred extension
+         */
+        std::vector<AString> readFileExtensions;
+        std::vector<AString> writeFileExtensions;
+        getWorkbenchSupportedImageFileExtensions(readFileExtensions,
+                                                 writeFileExtensions,
+                                                 ext);
+    }
+    
     return ext;
 }
 
@@ -801,8 +906,8 @@ DataFileTypeEnum::isValidFileExtension(const AString& filename,
      * See if filename ends with any of the available extensions for the
      * given data file type.
      */
-    for (std::vector<AString>::const_iterator iter = enumInstance->fileExtensions.begin();
-         iter != enumInstance->fileExtensions.end();
+    for (std::vector<AString>::const_iterator iter = enumInstance->readAndWriteFileExtensions.begin();
+         iter != enumInstance->readAndWriteFileExtensions.end();
          iter++) {
         const AString ext = ("." + *iter);
         if (filename.endsWith(ext)) {
@@ -835,7 +940,7 @@ DataFileTypeEnum::fromFileExtension(const AString& filename, bool* isValidOut)
          iter != enumData.end();
          iter++) {
         const DataFileTypeEnum& d = *iter;
-        const std::vector<AString> extensions = iter->fileExtensions;
+        const std::vector<AString> extensions = iter->readAndWriteFileExtensions;
         for (std::vector<AString>::const_iterator extIter = extensions.begin();
              extIter != extensions.end();
              extIter++) {
@@ -925,7 +1030,7 @@ DataFileTypeEnum::getAllEnums(std::vector<DataFileTypeEnum::Enum>& allEnums,
     const bool includeVolumeDynamicFlag = (options & OPTIONS_INCLUDE_VOLUME_DENSE_DYNAMIC);
     const bool includeUnknownFlag       = (options & OPTIONS_INCLUDE_UNKNOWN);
     
-    for (const auto dataType : enumData) {
+    for (const auto& dataType : enumData) {
         bool addEnumFlag(true);
         
         switch (dataType.enumValue) {
@@ -1048,4 +1153,246 @@ DataFileTypeEnum::getAllConnectivityEnums(std::vector<Enum>& connectivityEnumsOu
     
 }
 
+/**
+ * Create a QDialog file filter using the given file type name and extensions
+ * @param fileTypeName
+ *    Description of file type name
+ * @param fileExtensions
+ *    Valid file extensions
+ */
+AString
+DataFileTypeEnum::createQFileDialogNameFilter(const AString& fileTypeName,
+                                              const std::vector<AString>& fileExtensions)
+{
+    AString filterText(fileTypeName + " Files (");
+    
+    bool firstTime(true);
+    for (auto& ext : fileExtensions) {
+        if ( ! firstTime) {
+            filterText += " ";
+        }
+        filterText += ("*." + ext);
+        
+        firstTime = false;
+    }
+    filterText += ")";
+    
+    return filterText;
+}
+
+/**
+ * Get all image file extensions supported by Qt for reading and writing image files.
+ * @param readableExtensionsOut
+ *    Output contains all readable image file extensions
+ * @param writableExtensionsOut
+ *    Output contains all writable image file extensions
+ */
+void
+DataFileTypeEnum::getQtSupportedImageFileExtensions(std::vector<AString>& readableExtensionsOut,
+                                                    std::vector<AString>& writableExtensionsOut)
+{
+    readableExtensionsOut.clear();
+    writableExtensionsOut.clear();
+    
+    /*
+     * Extensions Qt can read (use set so extensions sorted)
+     */
+    std::set<AString> qtReadExtensions;
+    QList<QByteArray> readTypes(QImageReader::supportedImageFormats());
+    QListIterator<QByteArray> readTypesIterator(readTypes);
+    while (readTypesIterator.hasNext()) {
+        QString fmt(readTypesIterator.next());
+        qtReadExtensions.insert(fmt);
+    }
+    
+    /*
+     * Extensions Qt can write
+     */
+    std::set<AString> qtWriteExtensions;
+    QList<QByteArray> writeTypes(QImageWriter::supportedImageFormats());
+    QListIterator<QByteArray> writeTypesIterator(writeTypes);
+    while (writeTypesIterator.hasNext()) {
+        QString fmt(writeTypesIterator.next());
+        qtWriteExtensions.insert(fmt);
+    }
+    
+    readableExtensionsOut.insert(readableExtensionsOut.end(),
+                                 qtReadExtensions.begin(),
+                                 qtReadExtensions.end());
+    
+    writableExtensionsOut.insert(writableExtensionsOut.end(),
+                                 qtWriteExtensions.begin(),
+                                 qtWriteExtensions.end());
+}
+
+/**
+ * Get all image file extensions supported by Workbench for reading and writing image files.
+ * These are a subset of the extensions supported by Qt.
+ * @param readableExtensionsOut
+ *    Output contains all readable image file extensions
+ * @param writableExtensionsOut
+ *    Output contains all writable image file extensions
+ * @param defaultWritableExtension
+ *    Default extension for writing images
+ */
+void
+DataFileTypeEnum::getWorkbenchSupportedImageFileExtensions(std::vector<AString>& readableExtensionsOut,
+                                                           std::vector<AString>& writableExtensionsOut,
+                                                           AString& defaultWritableExtension)
+{
+    readableExtensionsOut.clear();
+    writableExtensionsOut.clear();
+    defaultWritableExtension.clear();
+    
+    /*
+     * Extensions that we want to support
+     * There are some such as SVG that we don't want to support
+     */
+    std::set<AString> supportedExtensions;
+    supportedExtensions.insert("bmp");
+    supportedExtensions.insert("gif");
+    supportedExtensions.insert("jpg");
+    supportedExtensions.insert("jpeg");
+    supportedExtensions.insert("jp2");
+    supportedExtensions.insert("png");
+    supportedExtensions.insert("ppm");
+    supportedExtensions.insert("tiff");
+    supportedExtensions.insert("tif");
+    
+    std::vector<AString> qtReadExtensions;
+    std::vector<AString> qtWriteExtensions;
+    getQtSupportedImageFileExtensions(qtReadExtensions,
+                                      qtWriteExtensions);
+    
+    /*
+     * Use only those extensions preferred for reading and writing
+     */
+    for (auto& ext : supportedExtensions) {
+        if (std::find(qtReadExtensions.begin(),
+                      qtReadExtensions.end(),
+                      ext) != qtReadExtensions.end()) {
+            readableExtensionsOut.push_back(ext);
+        }
+        if (std::find(qtWriteExtensions.begin(),
+                      qtWriteExtensions.end(),
+                      ext) != qtWriteExtensions.end()) {
+            writableExtensionsOut.push_back(ext);
+        }
+    }
+    
+    AString pngExtension;
+    AString jpegExtension;
+    AString jpgExtension;
+    AString tifExtension;
+    AString tiffExtension;
+    
+    for (auto& extension : writableExtensionsOut) {
+        if (extension == "png") {
+            pngExtension = extension;
+        }
+        else if (extension == "jpg") {
+            jpgExtension = extension;
+        }
+        else if (extension == "jpeg") {
+            jpegExtension = extension;
+        }
+        else if (extension == "tif") {
+            tifExtension = extension;
+        }
+        else if (extension == "tiff") {
+            tiffExtension = extension;
+        }
+    }
+
+    if ( ! writableExtensionsOut.empty()) {
+        defaultWritableExtension = writableExtensionsOut[0];
+        
+        if ( ! pngExtension.isEmpty()) {
+            defaultWritableExtension = pngExtension;
+        }
+        else if ( ! jpgExtension.isEmpty()) {
+            defaultWritableExtension = jpgExtension;
+        }
+        else if ( ! jpegExtension.isEmpty()) {
+            defaultWritableExtension = jpegExtension;
+        }
+        else if ( ! tiffExtension.isEmpty()) {
+            defaultWritableExtension = tiffExtension;
+        }
+        else if ( ! tifExtension.isEmpty()) {
+            defaultWritableExtension = tifExtension;
+        }
+    }
+}
+
+/**
+ * Get filters for saving images with each image type in its own filter and the default filter
+ * @param imageFiltersOut
+ *    All image filters to saving images
+ * @param defaultImageFilterOut
+ *    Default image filter
+ */
+void
+DataFileTypeEnum::getSaveQFileDialogImageFilters(std::vector<AString>& imageFiltersOut,
+                                                 AString& defaultImageFilterOut)
+{
+    imageFiltersOut.clear();
+    defaultImageFilterOut.clear();
+    
+    std::vector<AString> readableExtensions;
+    std::vector<AString> writableExtensions;
+    AString defaultWritableExtension;
+    getWorkbenchSupportedImageFileExtensions(readableExtensions,
+                                             writableExtensions,
+                                             defaultWritableExtension);
+    
+    const DataFileTypeEnum* imageType = findData(DataFileTypeEnum::IMAGE);
+    CaretAssert(imageType);
+    
+    for (const auto& extension: writableExtensions) {
+        const AString name(extension.toUpper()
+                           + " "
+                           + imageType->guiName);
+        const std::vector<AString> extVector { extension };
+        
+        const AString filter(createQFileDialogNameFilter(name,
+                                                         extVector));
+        imageFiltersOut.push_back(filter);
+
+        if (extension == defaultWritableExtension) {
+            defaultImageFilterOut = filter;
+        }
+    }
+    
+    if (defaultImageFilterOut.isEmpty()) {
+        if ( ! imageFiltersOut.empty()) {
+            defaultImageFilterOut = imageFiltersOut[0];
+        }
+    }
+}
+
+/**
+ * Get files extensions supports by QMovie
+ * @param readableExtensionsOut
+ *   Output containing extensions of files that can be read by QMovie
+ */
+void
+DataFileTypeEnum::getQtSupportedMovieFileExtensions(std::vector<AString>& readableExtensionsOut)
+{
+    /*
+     * Extensions Qt can read (use set so extensions sorted)
+     */
+    std::set<AString> qtReadExtensions;
+    QList<QByteArray> readTypes(QMovie::supportedFormats());
+    QListIterator<QByteArray> readTypesIterator(readTypes);
+    while (readTypesIterator.hasNext()) {
+        QString fmt(readTypesIterator.next());
+        qtReadExtensions.insert(fmt);
+    }
+
+    readableExtensionsOut.clear();
+    readableExtensionsOut.insert(readableExtensionsOut.end(),
+                                 qtReadExtensions.begin(),
+                                 qtReadExtensions.end());
+}
 
