@@ -39,7 +39,10 @@ public:
         DEVELOPER_FLAG_FLIP_PALETTE_NOT_DATA,
         DEVELOPER_FLAG_TEXTURE_VOLUME,
         DELELOPER_FLAG_VOXEL_SMOOTH,
-        DEVELOPER_FLAG_BALSA
+        DEVELOPER_FLAG_BALSA,
+        DEVELOPER_FLAG_VOXEL_CUBES_TEST,
+        DEVELOPER_FLAG_CHART_OPENGL_LINES,
+        DEVELOPER_FLAG_BLENDING
     };
 
     ~DeveloperFlagsEnum();
