@@ -24,6 +24,7 @@
 #undef __CARET_DATA_FILE_HELPER_DECLARE__
 
 #include "AnnotationFile.h"
+#include "AnnotationTextSubstitutionFile.h"
 #include "BorderFile.h"
 #include "CaretAssert.h"
 #include "CaretLogger.h"
@@ -307,6 +308,9 @@ CaretDataFileHelper::createCaretDataFileForFileType(const DataFileTypeEnum::Enum
         case DataFileTypeEnum::ANNOTATION:
             caretDataFile = new AnnotationFile();
             break;
+        case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+            caretDataFile = new AnnotationTextSubstitutionFile();
+            break;
         case DataFileTypeEnum::BORDER:
             caretDataFile = new BorderFile();
             break;
@@ -364,6 +368,9 @@ CaretDataFileHelper::createCaretDataFileForFileType(const DataFileTypeEnum::Enum
         case DataFileTypeEnum::METRIC:
             caretDataFile = new MetricFile();
             break;
+        case DataFileTypeEnum::METRIC_DYNAMIC:
+            CaretAssertMessage(0, "Never create a metric dynamic file");
+            break;
         case DataFileTypeEnum::PALETTE:
             caretDataFile = new PaletteFile();
             break;
@@ -385,6 +392,9 @@ CaretDataFileHelper::createCaretDataFileForFileType(const DataFileTypeEnum::Enum
         case DataFileTypeEnum::VOLUME:
             caretDataFile = new VolumeFile();
             break;
+        case DataFileTypeEnum::VOLUME_DYNAMIC:
+            CaretAssertMessage(0, "Never create a Volume Dynamic file");
+            break;
     }
     
     return caretDataFile;
