@@ -127,6 +127,12 @@ namespace caret {
         CaretPointer<VolumeFileEditorDelegate> m_volumeFileEditorDelegate;
         
         static const AString s_paletteColorMappingNameInMetaData;
+
+        int16_t m_writingDType;
+
+        bool m_writingDoScale;
+
+        double m_minScalingVal, m_maxScalingVal;
         
     protected:
         VolumeFile(const DataFileTypeEnum::Enum dataFileType);
@@ -200,6 +206,10 @@ namespace caret {
 
         virtual void writeFile(const AString& filename);
 
+        ///data type and scaling options
+        void setWritingDataTypeNoScaling(const int16_t& type = NIFTI_TYPE_FLOAT32);
+        void setWritingDataTypeAndScaling(const int16_t& type, const double& minval, const double& maxval);
+
         bool isEmpty() const { return VolumeBase::isEmpty(); }
         
         bool hasGoodSpatialInformation() const;
@@ -379,6 +389,7 @@ namespace caret {
        
         virtual bool getVolumeVoxelIdentificationForMaps(const std::vector<int32_t>& mapIndices,
                                                          const float xyz[3],
+                                                         const AString& dataValueSeparator,
                                                          int64_t ijkOut[3],
                                                          AString& textOut) const;
         
