@@ -846,7 +846,7 @@ GiftiFile::readFile(const AString& filename)
     this->setFileName(filename);
     
     GiftiFileSaxReader saxReader(this);
-    std::auto_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
+    std::unique_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
     try {
         parser->parseFile(filename, &saxReader);
     }
