@@ -27,11 +27,14 @@
 
 #include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "EventManager.h"
+#include "EventOpenGLObjectToWindowTransform.h"
 #include "GraphicsEngineDataOpenGL.h"
 #include "GraphicsPrimitive.h"
 #include "GraphicsPrimitiveV3f.h"
 #include "GraphicsPrimitiveV3fC4ub.h"
 #include "GraphicsPrimitiveV3fN3f.h"
+#include "GraphicsUtilitiesOpenGL.h"
 #include "MathFunctions.h"
 #include "Matrix4x4.h"
 
@@ -481,44 +484,7 @@ GraphicsShape::drawSphereByteColor(const float xyz[3],
     drawSpheresByteColor(xyz,
                          1,
                          rgba,
-                         diameter);
-    
-//    const int32_t numLatLonDivisions = 10;
-//    
-//    GraphicsPrimitive* spherePrimitive = NULL;
-//    for (const auto iter : s_byteSpherePrimitives) {
-//        const auto& key = iter.first;
-//        if ((key  == numLatLonDivisions)) {
-//            spherePrimitive = iter.second;
-//            CaretAssert(spherePrimitive);
-//            break;
-//        }
-//    }
-//    
-//    if (spherePrimitive == NULL) {
-//        const bool useStripsFlag = true;
-//        if (useStripsFlag) {
-//            spherePrimitive = createSpherePrimitiveTriangleStrips(numLatLonDivisions);
-//        }
-//        else {
-//            spherePrimitive = createSpherePrimitiveTriangles(numLatLonDivisions);
-//        }
-//        /* colors may change but not coordinates/normals */
-//        spherePrimitive->setUsageTypeAll(GraphicsPrimitive::UsageType::MODIFIED_ONCE_DRAWN_MANY_TIMES);
-//        spherePrimitive->setUsageTypeColors(GraphicsPrimitive::UsageType::MODIFIED_MANY_DRAWN_MANY_TIMES);
-//        s_byteSpherePrimitives.insert(std::make_pair(numLatLonDivisions,
-//                                                     spherePrimitive));
-//    }
-//
-//    CaretAssert(spherePrimitive);
-//    
-//    spherePrimitive->replaceAllVertexSolidByteRGBA(rgba);
-//    
-//    glPushMatrix();
-//    glTranslatef(xyz[0], xyz[1], xyz[2]);
-//    glScalef(diameter, diameter, diameter);
-//    GraphicsEngineDataOpenGL::draw(spherePrimitive);
-//    glPopMatrix();
+                         diameter);    
 }
 
 /**
@@ -543,7 +509,7 @@ GraphicsShape::drawSpheresByteColor(const float xyz[],
     const int32_t numLatLonDivisions = 10;
     
     GraphicsPrimitive* spherePrimitive = NULL;
-    for (const auto iter : s_byteSpherePrimitives) {
+    for (const auto& iter : s_byteSpherePrimitives) {
         const auto& key = iter.first;
         if ((key  == numLatLonDivisions)) {
             spherePrimitive = iter.second;
@@ -600,7 +566,7 @@ GraphicsShape::drawCircleFilled(const float xyz[3],
     const int32_t numberOfDivisions = 20;
     
     GraphicsPrimitive* circlePrimitive = NULL;
-    for (const auto keyPrim : s_byteCirclePrimitives) {
+    for (const auto& keyPrim : s_byteCirclePrimitives) {
         if (keyPrim.first == numberOfDivisions) {
             circlePrimitive = keyPrim.second;
             break;
@@ -630,6 +596,189 @@ GraphicsShape::drawCircleFilled(const float xyz[3],
     glPopMatrix();
 }
 
+/**
+ * Draw a filled circle at the given XYZ coordinate using a diameter
+ * that is a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param diameterPercentageOfViewportHeight
+ *    Diameter of the circle.
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawCircleFilledPercentViewportHeight(const float xyz[3],
+                                                     const uint8_t rgba[4],
+                                                     const float diameterPercentageOfViewportHeight,
+                                                     std::array<float, 3>* windowXYZOut)
+{
+    const float unusedSecondDiameter(0.0);
+    drawShapePercentViewportHeight(xyz,
+                                   rgba,
+                                   Shape::CIRCLE_FILLED,
+                                   diameterPercentageOfViewportHeight,
+                                   unusedSecondDiameter,
+                                   windowXYZOut);
+}
+
+/**
+ * Draw a ring at the given XYZ coordinate using sizes
+ * that are a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param innerDiameterPercentageOfViewportHeight
+ *    Inner diameter of the ring.
+ * @param outerDiameterPercentageOfViewportHeight
+ *    Outer diameter of the ring.
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawRingPercentViewportHeight(const float xyz[3],
+                                             const uint8_t rgba[4],
+                                             const float innerDiameterPercentageOfViewportHeight,
+                                             const float outerDiameterPercentageOfViewportHeight,
+                                             std::array<float, 3>* windowXYZOut)
+{
+    drawShapePercentViewportHeight(xyz,
+                                   rgba,
+                                   Shape::RING,
+                                   innerDiameterPercentageOfViewportHeight,
+                                   outerDiameterPercentageOfViewportHeight,
+                                   windowXYZOut);
+}
+
+/**
+ * Draw a filled circle at the given XYZ coordinate using a diameter
+ * that is a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param diameterPercentageOfViewportHeight
+ *    Diameter of the circle.
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawSquarePercentViewportHeight(const float xyz[3],
+                                               const uint8_t rgba[4],
+                                               const float diameterPercentageOfViewportHeight,
+                                               std::array<float, 3>* windowXYZOut)
+{
+    const float unusedSecondDiameter(0.0);
+    drawShapePercentViewportHeight(xyz,
+                                   rgba,
+                                   Shape::SQUARE,
+                                   diameterPercentageOfViewportHeight,
+                                   unusedSecondDiameter,
+                                   windowXYZOut);
+}
+
+/**
+ * Draw a shape at the given XYZ coordinate using a diameter
+ * that is a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param shape
+ *    Shape that is drawn
+ * @param diameterPercentageOfViewportHeight
+ *    Diameter of the shape.
+ * @param diameterTwoPercentageOfViewportHeight
+ *    Diameter of the shape for (outer for ring, unused by others).
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawShapePercentViewportHeight(const float xyz[3],
+                                              const uint8_t rgba[4],
+                                              const Shape shape,
+                                              const float diameterPercentageOfViewportHeight,
+                                              const float diameterTwoPercentageOfViewportHeight,
+                                              std::array<float, 3>* windowXYZOut)
+{
+    /*
+     * Invalidate optional output window coordinate
+     */
+    if (windowXYZOut != NULL) {
+        windowXYZOut->at(0) = -1.0f;
+    }
+    
+    /*
+     * The scale of X and Y may be very different and drawing a shape
+     * in model space requires scaling the circle separately in X and Y.
+     * It is easier to draw the shape in window space.
+     */
+    EventOpenGLObjectToWindowTransform xform(EventOpenGLObjectToWindowTransform::SpaceType::MODEL);
+    EventManager::get()->sendEvent(xform.getPointer());
+    if (xform.isValid()) {
+        float windowXYZ[3];
+        xform.transformPoint(xyz,
+                             windowXYZ);
+        
+        std::array<int32_t, 4> vp = xform.getViewport();
+        
+        glMatrixMode(GL_PROJECTION);
+        glPushMatrix();
+        glLoadIdentity();
+        glOrtho(vp[0], vp[0] + vp[2],
+                vp[1], vp[1] + vp[3],
+                -10.0, 10.0);
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
+        glLoadIdentity();
+        
+        const float pixelSize = GraphicsUtilitiesOpenGL::convertPercentageOfViewportHeightToPixels(diameterPercentageOfViewportHeight);
+        const float pixelSizeTwo = GraphicsUtilitiesOpenGL::convertPercentageOfViewportHeightToPixels(diameterTwoPercentageOfViewportHeight);
+
+        switch (shape) {
+            case Shape::CIRCLE_FILLED:
+                GraphicsShape::drawCircleFilled(windowXYZ,
+                                                rgba,
+                                                pixelSize);
+                break;
+            case Shape::SQUARE:
+                GraphicsShape::drawSquare(windowXYZ,
+                                          rgba,
+                                          pixelSize);
+                break;
+            case Shape::RING:
+                GraphicsShape::drawRing(windowXYZ,
+                                        rgba,
+                                        pixelSize / 2,
+                                        pixelSizeTwo / 2);
+                break;
+        }
+        
+        if (windowXYZOut != NULL) {
+            windowXYZOut->at(0) = windowXYZ[0];
+            windowXYZOut->at(1) = windowXYZ[1];
+            windowXYZOut->at(2) = windowXYZ[2];
+        }
+
+        
+        glPopMatrix();
+        glMatrixMode(GL_PROJECTION);
+        glPopMatrix();
+        glMatrixMode(GL_MODELVIEW);
+    }
+}
+
+
 /**
  * Draw a squares at the given XYZ coordinates
  *
@@ -784,7 +933,7 @@ GraphicsShape::drawRing(const float xyz[3],
     const int32_t numberOfDivisions = 20;
     
     GraphicsPrimitive* ringPrimitive = NULL;
-    for (const auto keyPrim : s_byteRingPrimitives) {
+    for (const auto& keyPrim : s_byteRingPrimitives) {
         if (keyPrim.first.matches(numberOfDivisions,
                                   innerRadius,
                                   outerRadius)) {
@@ -829,7 +978,6 @@ GraphicsShape::createCirclePrimitive(const int32_t numberOfDivisions,
     /*
      * Setup step size based upon number of points around circle
      */
-    //    const int32_t numberOfPoints = 8;
     const float step = (2.0 * M_PI) / numberOfDivisions;
     
     const uint8_t rgba[] { 255, 255, 255, 255 };
