@@ -305,7 +305,8 @@ AnnotationTextAlignmentWidget::horizontalAlignmentActionSelected(QAction* action
                                                     annotations);
         AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
         AString errorMessage;
-        if ( ! annMan->applyCommand(undoCommand,
+        if ( ! annMan->applyCommand(UserInputModeEnum::Enum::ANNOTATIONS,
+                                    undoCommand,
                                     errorMessage)) {
             WuQMessageBox::errorOk(this,
                                    errorMessage);
@@ -339,7 +340,8 @@ AnnotationTextAlignmentWidget::verticalAlignmentActionSelected(QAction* action)
                                                   annotations);
         AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
         AString errorMessage;
-        if ( ! annMan->applyCommand(undoCommand,
+        if ( ! annMan->applyCommand(UserInputModeEnum::Enum::ANNOTATIONS,
+                                    undoCommand,
                                     errorMessage)) {
             WuQMessageBox::errorOk(this,
                                    errorMessage);
