@@ -28,6 +28,7 @@
 #include "EventListenerInterface.h"
 #include "SceneableInterface.h"
 
+class QSplitter;
 class QTabWidget;
 
 namespace caret {
@@ -39,8 +40,10 @@ namespace caret {
     class CiftiConnectivityMatrixViewController;
     class FiberOrientationSelectionViewController;
     class FociSelectionViewController;
+    class IdentificationDisplayWidget;
     class ImageSelectionViewController;
     class LabelSelectionViewController;
+    class MediaOverlaySetViewController;
     class OverlaySetViewController;
     class VolumeSurfaceOutlineSetViewController;
     class WuQTabWidgetWithSizeHint;
@@ -89,6 +92,8 @@ namespace caret {
         int addToTabWidget(QWidget* page,
                            const QString& label);
         
+        QWidget* createSplitterAndIdentificationWidget(const Qt::Orientation orientation);
+        
         OverlaySetViewController* m_overlaySetViewController;
         
         AnnotationSelectionViewController* m_annotationViewController;
@@ -111,6 +116,8 @@ namespace caret {
         
         LabelSelectionViewController* m_labelSelectionViewController;
         
+        MediaOverlaySetViewController* m_mediaSelectionViewController;
+        
         VolumeSurfaceOutlineSetViewController* m_volumeSurfaceOutlineSetViewController;
         
         WuQTabWidgetWithSizeHint* m_tabWidget;
@@ -141,8 +148,14 @@ namespace caret {
         
         int32_t m_labelTabIndex;
         
+        int32_t m_mediaTabIndex;
+        
         int32_t m_volumeSurfaceOutlineTabIndex;
         
+        QSplitter* m_splitterWidget = NULL;
+        
+        IdentificationDisplayWidget* m_identificationWidget = NULL;
+        
         int m_sizeHintWidth = -1;
         
         int m_sizeHintHeight = -1;
