@@ -28,7 +28,13 @@
 #undef __CARET_FILE_DIALOG_DECLARE__
 
 #include "Brain.h"
+#include "CaretAssert.h"
+#include "CaretPreferences.h"
 #include "GuiManager.h"
+#include "RecentFileItem.h"
+#include "RecentFileItemsFilter.h"
+#include "RecentFileItemsContainer.h"
+#include "SessionManager.h"
 
 using namespace caret;
 
@@ -97,6 +103,19 @@ FilterFilesProxyModel::setDataFileTypeForFiltering(const DataFileTypeEnum::Enum
     m_dataFileType = dataFileType;
 }
 
+/**
+ * Must override sort() and then call the model's sort().  Without this, columns sort as strings
+ * and date/size sorting does not work properly.
+ *
+ * From: https://stackoverflow.com/questions/53789779/qfiledialog-with-proxy-model-sort-by-date-actually-uses-alphabetical-order-of-da
+ */
+void
+FilterFilesProxyModel::sort(int column, Qt::SortOrder order)
+{
+    sourceModel()->sort(column,
+                        order);
+}
+
 /**
  * \class caret::CaretFileDialog 
  * \brief Adds additional functionality over Qt's QFileDialog.
@@ -104,27 +123,47 @@ FilterFilesProxyModel::setDataFileTypeForFiltering(const DataFileTypeEnum::Enum
 
 /**
  * Constructor.
+ * @param mode
+ *    The mode for open/save
+ * @param parent
+ *    Parent on which this dialog is displayed.
+ * @param f
+ *    Qt's window flags
  */
-CaretFileDialog::CaretFileDialog(QWidget* parent,
+CaretFileDialog::CaretFileDialog(const Mode mode,
+                                 QWidget* parent,
                                  Qt::WindowFlags f)
 : QFileDialog(parent,
-                f)
+                f),
+m_mode(mode)
 {
     this->initializeCaretFileDialog();
         
     this->setDirectory(GuiManager::get()->getBrain()->getCurrentDirectory());
 }
+
 /**
  * Constructor.
+ * @param mode
+ *    The mode for open/save
+ * @param parent
+ *    Parent on which this dialog is displayed.
+ * @param caption
+ *    Caption for dialog (if not provided a default caption is shown)
+ * @param directory
+ *    Directory show by dialog (Brain's current directory if empty string)
+ * @param filter
  */
-CaretFileDialog::CaretFileDialog(QWidget* parent,
+CaretFileDialog::CaretFileDialog(const Mode mode,
+                                 QWidget* parent,
                                  const QString& caption,
                                  const QString& directory,
                                  const QString& filter)
 : QFileDialog(parent,
               caption,
               directory,
-              filter)
+              filter),
+m_mode(mode)
 {
     this->initializeCaretFileDialog();
     
@@ -176,6 +215,25 @@ CaretFileDialog::initializeCaretFileDialog()
     
     QObject::connect(this, SIGNAL(filterSelected(const QString&)),
                      this, SLOT(fileFilterWasChanged(const QString&)));
+    
+    /*
+     * Add recent directories to history
+     */
+    CaretPreferences* prefs = SessionManager::get()->getCaretPreferences();
+    std::vector<AString> recentDirectories;
+    const bool favoritesFirstFlag(false);
+    prefs->getRecentDirectoriesForOpenFileDialogHistory(favoritesFirstFlag,
+                                                        recentDirectories);
+    const int32_t displayCount = std::min(static_cast<int32_t>(recentDirectories.size()),
+                                          10);
+    
+    QStringList historyList;
+    for (int32_t i = 0; i < displayCount; i++) {
+        CaretAssertVectorIndex(recentDirectories, i);
+        historyList.push_front(recentDirectories[i]);
+    }
+    
+    setHistory(historyList);
 }
 
 /**
@@ -243,7 +301,8 @@ CaretFileDialog::getOpenFileNameDialog(QWidget *parent,
                                      QString *selectedFilter,
                                      Options options )
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_OPEN,
+                        parent,
                         caption,
                         dir,
                         filter);
@@ -291,11 +350,12 @@ CaretFileDialog::getOpenFileNameDialog(const DataFileTypeEnum::Enum dataFileType
                                        Options options)
 {
     
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_OPEN,
+                        parent,
                         caption,
                         dir,
-                        DataFileTypeEnum::toQFileDialogFilter(dataFileType));
-    cfd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
+                        DataFileTypeEnum::toQFileDialogFilterForReading(dataFileType));
+    cfd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilterForReading(dataFileType));
     cfd.setOptions(options);
     cfd.setAcceptMode(CaretFileDialog::AcceptOpen);
     cfd.setFileMode(CaretFileDialog::AnyFile);
@@ -338,7 +398,8 @@ CaretFileDialog::getSaveFileNameDialog(QWidget *parent,
                                      QString *selectedFilter,
                                      Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_SAVE,
+                        parent,
                         caption,
                         dir,
                         filter);
@@ -389,11 +450,12 @@ CaretFileDialog::getSaveFileNameDialog(const DataFileTypeEnum::Enum dataFileType
                                        const QString &dir,
                                        Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_SAVE,
+                        parent,
                         caption,
                         dir,
-                        DataFileTypeEnum::toQFileDialogFilter(dataFileType));
-    cfd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
+                        DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
+    cfd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
     cfd.setOptions(options);
     cfd.setAcceptMode(QFileDialog::AcceptSave);
     cfd.setFileMode(CaretFileDialog::AnyFile);
@@ -441,11 +503,12 @@ CaretFileDialog::getChooseFileNameDialog(const DataFileTypeEnum::Enum dataFileTy
                                          const QString& directoryOrFileName,
                                          QWidget *parent)
 {
-    CaretFileDialog fd(parent);
+    CaretFileDialog fd(Mode::MODE_SAVE,
+                       parent);
     if (dataFileType != DataFileTypeEnum::UNKNOWN) {
-        fd.setNameFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
+        fd.setNameFilter(DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
     }
-    fd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
+    fd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
     fd.setAcceptMode(CaretFileDialog::AcceptSave);
     fd.setFileMode(CaretFileDialog::AnyFile);
     fd.setViewMode(CaretFileDialog::List);
@@ -502,7 +565,8 @@ CaretFileDialog::getExistingDirectoryDialog(QWidget *parent,
                                           const QString &dir,
                                           Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_SAVE,
+                        parent,
                         caption,
                         dir,
                         "");
@@ -549,7 +613,8 @@ CaretFileDialog::getOpenFileNamesDialog(QWidget *parent,
                                           QString *selectedFilter,
                                           Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_OPEN,
+                        parent,
                         caption,
                         dir,
                         filter);
