@@ -26,6 +26,7 @@
 #include <QCheckBox>
 #include <QComboBox>
 #include <QGridLayout>
+#include <QGroupBox>
 #include <QBoxLayout>
 #include <QHBoxLayout>
 #include <QLabel>
@@ -36,14 +37,17 @@
 #include "BrainConstants.h"
 #include "BrowserTabContent.h"
 #include "CaretAssert.h"
+#include "CaretColorToolButton.h"
 #include "ChartTwoOverlay.h"
 #include "ChartTwoOverlaySet.h"
 #include "ChartTwoOverlayViewController.h"
+#include "EnumComboBoxTemplate.h"
 #include "EventGraphicsUpdateOneWindow.h"
 #include "EventManager.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GuiManager.h"
 #include "MapYokingGroupComboBox.h"
+#include "WuQDoubleSpinBox.h"
 #include "WuQGridLayoutGroup.h"
 #include "WuQtUtilities.h"
 
@@ -102,26 +106,24 @@ m_browserWindowIndex(browserWindowIndex)
     }
     
     if (orientation == Qt::Horizontal) {
-        static const int COLUMN_ON = 0;
-        static const int COLUMN_LOAD = 1;
-        static const int COLUMN_SETTINGS_EDIT = 2;
-        static const int COLUMN_SETTINGS_COLOR_BAR = 3;
-        static const int COLUMN_SETTINGS_CONSTRUCTION = 4;
-        static const int COLUMN_MATRIX_VIEW = 5;
-        static const int COLUMN_AXIS = 6;
-        static const int COLUMN_FILE = 7;
-        static const int COLUMN_YOKE = 8;
-        static const int COLUMN_MAP_INDEX = 9;
-        static const int COLUMN_ALL_MAPS = 10;
-        static const int COLUMN_MAP_NAME = 11;
+        int32_t columnCounter(0);
+        static const int COLUMN_ON         = columnCounter++;
+        static const int COLUMN_LOAD       = columnCounter++;
+        static const int COLUMN_SETTINGS   = columnCounter++;
+        static const int COLUMN_LINE_WIDTH = columnCounter++;
+        static const int COLUMN_OPACITY    = columnCounter++;
+        static const int COLUMN_POINT      = columnCounter++;
+        static const int COLUMN_FILE       = columnCounter++;
+        static const int COLUMN_YOKE       = columnCounter++;
+        static const int COLUMN_MAP_INDEX  = columnCounter++;
+        static const int COLUMN_ALL_MAPS   = columnCounter++;
+        static const int COLUMN_MAP_NAME   = columnCounter++;
         
         gridLayout->setColumnStretch(COLUMN_ON, 0);
         gridLayout->setColumnStretch(COLUMN_LOAD, 0);
-        gridLayout->setColumnStretch(COLUMN_SETTINGS_EDIT, 0);
-        gridLayout->setColumnStretch(COLUMN_SETTINGS_COLOR_BAR, 0);
-        gridLayout->setColumnStretch(COLUMN_SETTINGS_CONSTRUCTION, 0);
-        gridLayout->setColumnStretch(COLUMN_MATRIX_VIEW, 0);
-        gridLayout->setColumnStretch(COLUMN_AXIS, 0);
+        gridLayout->setColumnStretch(COLUMN_SETTINGS, 0);
+        gridLayout->setColumnStretch(COLUMN_OPACITY, 0);
+        gridLayout->setColumnStretch(COLUMN_LINE_WIDTH, 0);
         gridLayout->setColumnStretch(COLUMN_FILE, 100);
         gridLayout->setColumnStretch(COLUMN_YOKE, 0);
         gridLayout->setColumnStretch(COLUMN_ALL_MAPS, 0);
@@ -129,36 +131,60 @@ m_browserWindowIndex(browserWindowIndex)
         gridLayout->setColumnStretch(COLUMN_MAP_NAME, 100);
         
         QLabel* onLabel       = new QLabel("On");
-        QLabel* loadLabel     = new QLabel("Load");
+        m_loadLabel           = new QLabel("Load");
         QLabel* settingsLabel = new QLabel("Settings");
+        m_opacityLabel        = new QLabel("Opacity");
+        m_lineWidthLabel      = new QLabel("Width");
+        m_pointLabel          = new QLabel("Point");
         QLabel* fileLabel     = new QLabel("File");
         QLabel* yokeLabel     = new QLabel("Yoke");
-        QLabel* allMapsLabel  = new QLabel("All");
+        m_allMapsLabel        = new QLabel("All");
         m_mapRowOrColumnIndexLabel = new QLabel("Index");
         m_mapRowOrColumnNameLabel  = new QLabel("Name");
         
         int row = gridLayout->rowCount();
         gridLayout->addWidget(onLabel, row, COLUMN_ON, Qt::AlignHCenter);
-        gridLayout->addWidget(loadLabel, row, COLUMN_LOAD, Qt::AlignHCenter);
-        gridLayout->addWidget(settingsLabel, row, COLUMN_SETTINGS_EDIT, 1, 5, Qt::AlignHCenter);
+        gridLayout->addWidget(m_loadLabel, row, COLUMN_LOAD, Qt::AlignHCenter);
+        gridLayout->addWidget(settingsLabel, row, COLUMN_SETTINGS, Qt::AlignHCenter);
+        gridLayout->addWidget(m_lineWidthLabel, row, COLUMN_LINE_WIDTH, Qt::AlignHCenter);
+        gridLayout->addWidget(m_opacityLabel, row, COLUMN_OPACITY, Qt::AlignHCenter);
+        gridLayout->addWidget(m_pointLabel, row, COLUMN_POINT, Qt::AlignHCenter);
         gridLayout->addWidget(fileLabel, row, COLUMN_FILE, Qt::AlignHCenter);
         gridLayout->addWidget(yokeLabel, row, COLUMN_YOKE, Qt::AlignHCenter);
-        gridLayout->addWidget(allMapsLabel, row, COLUMN_ALL_MAPS, Qt::AlignHCenter);
+        gridLayout->addWidget(m_allMapsLabel, row, COLUMN_ALL_MAPS, Qt::AlignHCenter);
         gridLayout->addWidget(m_mapRowOrColumnIndexLabel, row, COLUMN_MAP_INDEX, Qt::AlignHCenter);
         gridLayout->addWidget(m_mapRowOrColumnNameLabel, row, COLUMN_MAP_NAME, Qt::AlignHCenter);
         
         for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS; i++) {
-            row = gridLayout->rowCount();
             ChartTwoOverlayViewController* covc = m_chartOverlayViewControllers[i];
+
+            QWidget* settingsWidget = new QWidget();
+            QHBoxLayout* settingsLayout = new QHBoxLayout(settingsWidget);
+            settingsLayout->setContentsMargins(0, 0, 0, 0);
+            settingsLayout->setSpacing(3);
+            settingsLayout->addWidget(covc->m_settingsToolButton);
+            settingsLayout->addWidget(covc->m_colorBarToolButton);
+            settingsLayout->addWidget(covc->m_constructionToolButton);
+            settingsLayout->addWidget(covc->m_matrixTriangularViewModeToolButton);
+            settingsLayout->addWidget(covc->m_lineLayerColorToolButton);
+            settingsLayout->addWidget(covc->m_lineLayerToolTipOffsetToolButton);
+            settingsLayout->addWidget(covc->m_lineLayerNormalizationToolButton);
+
+            QWidget* pointWidget = new QWidget();
+            QHBoxLayout* pointLayout = new QHBoxLayout(pointWidget);
+            pointLayout->setContentsMargins(0, 0, 0, 0);
+            pointLayout->addWidget(covc->m_lineLayerActiveComboBox->getWidget());
+            pointLayout->addWidget(covc->m_selectedPointIndexSpinBox);
+            
+            row = gridLayout->rowCount();
             WuQGridLayoutGroup* glg = m_chartOverlayGridLayoutGroups[i];
             
             glg->addWidget(covc->m_enabledCheckBox, row, COLUMN_ON, Qt::AlignHCenter);
             glg->addWidget(covc->m_lineSeriesLoadingEnabledCheckBox, row, COLUMN_LOAD, Qt::AlignHCenter);
-            glg->addWidget(covc->m_settingsToolButton, row, COLUMN_SETTINGS_EDIT, Qt::AlignHCenter);
-            glg->addWidget(covc->m_colorBarToolButton, row, COLUMN_SETTINGS_COLOR_BAR, Qt::AlignHCenter);
-            glg->addWidget(covc->m_constructionToolButton, row, COLUMN_SETTINGS_CONSTRUCTION, Qt::AlignHCenter);
-            glg->addWidget(covc->m_matrixTriangularViewModeToolButton, row, COLUMN_MATRIX_VIEW, Qt::AlignHCenter);
-            glg->addWidget(covc->m_axisLocationToolButton, row, COLUMN_AXIS, Qt::AlignHCenter);
+            glg->addWidget(settingsWidget, row, COLUMN_SETTINGS, Qt::AlignHCenter);
+            glg->addWidget(covc->m_lineLayerWidthSpinBox->getWidget(), row, COLUMN_LINE_WIDTH);
+            glg->addWidget(covc->m_matrixOpacitySpinBox->getWidget(), row, COLUMN_OPACITY);
+            glg->addWidget(pointWidget, row, COLUMN_POINT);
             glg->addWidget(covc->m_mapFileComboBox, row, COLUMN_FILE);
             glg->addWidget(covc->m_mapRowOrColumnYokingGroupComboBox->getWidget(), row, COLUMN_YOKE, Qt::AlignHCenter);
             glg->addWidget(covc->m_allMapsCheckBox, row, COLUMN_ALL_MAPS, Qt::AlignHCenter);
@@ -167,29 +193,29 @@ m_browserWindowIndex(browserWindowIndex)
         }
     }
     else {
-        static const int ROW_ONE_COLUMN_ON = 0;
-        static const int ROW_ONE_COLUMN_SETTINGS_EDIT = 1;
-        static const int ROW_ONE_COLUMN_SETTINGS_COLOR_BAR = 2;
-        static const int ROW_ONE_COLUMN_SETTINGS_CONSTRUCTION = 3;
-        static const int ROW_ONE_COLUMN_MATRIX_VIEW = 4;
-        static const int ROW_ONE_COLUMN_AXIS = 5;
-        static const int ROW_ONE_COLUMN_FILE_LABEL = 6;
-        static const int ROW_ONE_COLUMN_FILE_COMBO_BOX = 7;
-        
-        static const int ROW_TWO_COLUMN_LOAD = 0;
-        static const int ROW_TWO_COLUMN_YOKE = 2;
-        static const int ROW_TWO_COLUMN_ALL_MAPS = 4;
-        static const int ROW_TWO_COLUMN_MAP_INDEX = 6;
-        static const int ROW_TWO_COLUMN_MAP_NAME = 7;
-        
-        gridLayout->setColumnStretch(0, 0);
-        gridLayout->setColumnStretch(1, 0);
-        gridLayout->setColumnStretch(2, 0);
-        gridLayout->setColumnStretch(3, 0);
-        gridLayout->setColumnStretch(4, 0);
-        gridLayout->setColumnStretch(5, 0);
-        gridLayout->setColumnStretch(6, 0);
-        gridLayout->setColumnStretch(7, 100);
+        int32_t columnCounter(0);
+        static const int COLUMN_ONE   = columnCounter++;
+        static const int COLUMN_TWO   = columnCounter++;
+        static const int COLUMN_THREE   = columnCounter++;
+
+        for (int32_t i = 0; i < columnCounter; i++) {
+            gridLayout->setColumnStretch(i, 0);
+        }
+        gridLayout->setColumnStretch(COLUMN_THREE, 100);
+        gridLayout->setVerticalSpacing(0);
+
+        const bool showTitlesFlag(true);
+        if (showTitlesFlag) {
+            m_pointLabel = new QLabel("Point");
+            
+            const int titleRow(gridLayout->rowCount());
+            gridLayout->addWidget(new QLabel("Settings"),
+                                  titleRow, COLUMN_ONE, Qt::AlignHCenter);
+            gridLayout->addWidget(m_pointLabel,
+                                  titleRow, COLUMN_TWO, Qt::AlignHCenter);
+            gridLayout->addWidget(new QLabel("Yoke / File / Map"),
+                                  titleRow, COLUMN_THREE, Qt::AlignLeft);
+        }
 
         for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS; i++) {
             WuQGridLayoutGroup* glg = m_chartOverlayGridLayoutGroups[i];
@@ -207,22 +233,49 @@ m_browserWindowIndex(browserWindowIndex)
 
             ChartTwoOverlayViewController* covc = m_chartOverlayViewControllers[i];
 
-            QLabel* fileLabel = new QLabel("File");
-            glg->addWidget(covc->m_enabledCheckBox, row, ROW_ONE_COLUMN_ON, Qt::AlignLeft);
-            glg->addWidget(covc->m_settingsToolButton, row, ROW_ONE_COLUMN_SETTINGS_EDIT, Qt::AlignHCenter);
-            glg->addWidget(covc->m_colorBarToolButton, row, ROW_ONE_COLUMN_SETTINGS_COLOR_BAR, Qt::AlignHCenter);
-            glg->addWidget(covc->m_constructionToolButton, row, ROW_ONE_COLUMN_SETTINGS_CONSTRUCTION, Qt::AlignHCenter);
-            glg->addWidget(covc->m_matrixTriangularViewModeToolButton, row, ROW_ONE_COLUMN_MATRIX_VIEW, Qt::AlignHCenter);
-            glg->addWidget(covc->m_axisLocationToolButton, row, ROW_ONE_COLUMN_AXIS, Qt::AlignHCenter);
-            glg->addWidget(fileLabel, row, ROW_ONE_COLUMN_FILE_LABEL, Qt::AlignHCenter);
-            glg->addWidget(covc->m_mapFileComboBox, row, ROW_ONE_COLUMN_FILE_COMBO_BOX, 1, 2);
-            row++;
+            QWidget* topLeftWidget = new QWidget();
+            QHBoxLayout* topLeftLayout = new QHBoxLayout(topLeftWidget);
+            topLeftLayout->setContentsMargins(0, 0, 0, 0);
+            topLeftLayout->setSpacing(3);
+            topLeftLayout->addWidget(covc->m_enabledCheckBox);
+            topLeftLayout->addWidget(covc->m_lineSeriesLoadingEnabledCheckBox);
+            topLeftLayout->addWidget(covc->m_lineLayerWidthSpinBox->getWidget());
+            topLeftLayout->addWidget(covc->m_matrixOpacitySpinBox->getWidget());
             
-            glg->addWidget(covc->m_lineSeriesLoadingEnabledCheckBox, row, ROW_TWO_COLUMN_LOAD, Qt::AlignLeft);
-            glg->addWidget(covc->m_mapRowOrColumnYokingGroupComboBox->getWidget(), row, ROW_TWO_COLUMN_YOKE, 1, 2, Qt::AlignHCenter);
-            glg->addWidget(covc->m_allMapsCheckBox, row, ROW_TWO_COLUMN_ALL_MAPS, 1, 2, Qt::AlignHCenter);
-            glg->addWidget(covc->m_mapRowOrColumnIndexSpinBox, row, ROW_TWO_COLUMN_MAP_INDEX, Qt::AlignHCenter);
-            glg->addWidget(covc->m_mapRowOrColumnNameComboBox, row, ROW_TWO_COLUMN_MAP_NAME);
+            QWidget* topRightWidget = new QWidget();
+            QHBoxLayout* topRightLayout = new QHBoxLayout(topRightWidget);
+            topRightLayout->setContentsMargins(0, 0, 0, 0);
+            topRightLayout->setSpacing(3);
+            topRightLayout->addWidget(covc->m_mapRowOrColumnYokingGroupComboBox->getWidget(), 0);
+            topRightLayout->addWidget(covc->m_mapFileComboBox, 100);
+            
+            QWidget* bottomLeftWidget = new QWidget();
+            QHBoxLayout* bottomLeftLayout = new QHBoxLayout(bottomLeftWidget);
+            bottomLeftLayout->setContentsMargins(0, 0, 0, 0);
+            bottomLeftLayout->setSpacing(3);
+            bottomLeftLayout->addWidget(covc->m_settingsToolButton);
+            bottomLeftLayout->addWidget(covc->m_colorBarToolButton);
+            bottomLeftLayout->addWidget(covc->m_constructionToolButton);
+            bottomLeftLayout->addWidget(covc->m_matrixTriangularViewModeToolButton);
+            bottomLeftLayout->addWidget(covc->m_lineLayerColorToolButton);
+            bottomLeftLayout->addWidget(covc->m_lineLayerToolTipOffsetToolButton);
+            bottomLeftLayout->addWidget(covc->m_lineLayerNormalizationToolButton);
+
+            QWidget* bottomRightWidget = new QWidget();
+            QHBoxLayout* bottomRightLayout = new QHBoxLayout(bottomRightWidget);
+            bottomRightLayout->setContentsMargins(0, 0, 0, 0);
+            bottomRightLayout->setSpacing(3);
+            bottomRightLayout->addWidget(covc->m_allMapsCheckBox, 0);
+            bottomRightLayout->addWidget(covc->m_mapRowOrColumnIndexSpinBox, 0);
+            bottomRightLayout->addWidget(covc->m_mapRowOrColumnNameComboBox, 100);
+            
+            glg->addWidget(topLeftWidget, row, COLUMN_ONE, Qt::AlignLeft);
+            glg->addWidget(covc->m_lineLayerActiveComboBox->getWidget(), row, COLUMN_TWO);
+            glg->addWidget(topRightWidget, row, COLUMN_THREE);
+            row++;
+            glg->addWidget(bottomLeftWidget, row, COLUMN_ONE, Qt::AlignLeft);
+            glg->addWidget(covc->m_selectedPointIndexSpinBox, row, COLUMN_TWO);
+            glg->addWidget(bottomRightWidget, row, COLUMN_THREE);
         }
     }
     
@@ -291,42 +344,65 @@ ChartTwoOverlaySetViewController::updateViewController()
     const int32_t numberOfDisplayedOverlays = chartOverlaySet->getNumberOfDisplayedOverlays();
     
     for (int32_t i = 0; i < numberOfOverlays; i++) {
+        bool showAllMapsLabelFlag(false);
+        bool showLoadLabelFlag(false);
+        bool showLineWidthLabelFlag(false);
+        bool showOpacityLabelFlag(false);
+        bool showPointLabelFlag(false);
+        switch (chartOverlaySet->getChartTwoDataType()) {
+            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_HISTOGRAM:
+                showAllMapsLabelFlag = true;
+                break;
+            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_INVALID:
+                break;
+            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_LAYER:
+                showLineWidthLabelFlag = true;
+                showPointLabelFlag     = true;
+                break;
+            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_SERIES:
+                showLoadLabelFlag = true;
+                break;
+            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_MATRIX:
+                showOpacityLabelFlag = true;
+                break;
+        }
+        if (m_allMapsLabel != NULL) {
+            m_allMapsLabel->setVisible(showAllMapsLabelFlag);
+        }
+        if (m_loadLabel != NULL) {
+            m_loadLabel->setVisible(showLoadLabelFlag);
+        }
+        if (m_lineWidthLabel != NULL) {
+            m_lineWidthLabel->setVisible(showLineWidthLabelFlag);
+        }
+        if (m_opacityLabel != NULL) {
+            m_opacityLabel->setVisible(showOpacityLabelFlag);
+        }
+        if (m_pointLabel != NULL) {
+            m_pointLabel->setVisible(showPointLabelFlag);
+        }
+
         ChartTwoOverlay* chartOverlay = NULL;
         if (chartOverlaySet != NULL) {
             chartOverlay = chartOverlaySet->getOverlay(i);
         }
-        m_chartOverlayViewControllers[i]->updateViewController(chartOverlay);
         
         bool displayOverlay = (chartOverlay != NULL);
         if (i >= numberOfDisplayedOverlays) {
             displayOverlay = false;
         }
+
         CaretAssertVectorIndex(m_chartOverlayGridLayoutGroups, i);
         m_chartOverlayGridLayoutGroups[i]->setVisible(displayOverlay);
+        
+        if (displayOverlay) {
+            /*
+             * Need to do AFTER setting visibility
+             */
+            CaretAssertVectorIndex(m_chartOverlayViewControllers, i);
+            m_chartOverlayViewControllers[i]->updateViewController(chartOverlay);
+        }
     }
-    
-//    const ChartTwoOverlay* primaryOverlay = chartOverlaySet->getPrimaryOverlay();
-//    if (primaryOverlay != NULL) {
-//        AString mapRowOrColumnName = "Map";
-//        switch (primaryOverlay->getChartTwoDataType()) {
-//            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_INVALID:
-//                break;
-//            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_HISTOGRAM:
-//                break;
-//            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_SERIES:
-//                break;
-//            case ChartTwoDataTypeEnum::CHART_DATA_TYPE_MATRIX:
-//                mapRowOrColumnName = "Row";
-//                break;
-//        }
-//        
-//        if (m_mapRowOrColumnIndexLabel != NULL) {
-//            m_mapRowOrColumnIndexLabel->setText(mapRowOrColumnName+ " Index");
-//        }
-//        if (m_mapRowOrColumnNameLabel != NULL) {
-//            m_mapRowOrColumnNameLabel->setText(mapRowOrColumnName + " Name");
-//        }
-//    }
 }
 
 /**
@@ -357,6 +433,7 @@ ChartTwoOverlaySetViewController::processAddOverlayAbove(const int32_t overlayIn
     if (chartOverlaySet != NULL) {
         chartOverlaySet->insertOverlayAbove(overlayIndex);
         this->updateColoringAndGraphics();
+        this->updateViewController();
     }
 }
 
@@ -372,6 +449,7 @@ ChartTwoOverlaySetViewController::processAddOverlayBelow(const int32_t overlayIn
     if (chartOverlaySet != NULL) {
         chartOverlaySet->insertOverlayBelow(overlayIndex);
         this->updateColoringAndGraphics();
+        updateViewController();
     }
 }
 
