@@ -35,12 +35,11 @@
 #include "ChartTwoOverlay.h"
 #include "CiftiFiberTrajectoryFile.h"
 #include "CiftiConnectivityMatrixParcelFile.h"
-#include "EventChartOverlayValidate.h"
+#include "EventChartTwoOverlayValidate.h"
 #include "EventDataFileDelete.h"
 #include "EventGraphicsUpdateAllWindows.h"
 #include "EventManager.h"
 #include "EventOverlayValidate.h"
-#include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GiftiLabelTableEditor.h"
 #include "MapSettingsChartTwoLineHistoryWidget.h"
@@ -121,7 +120,7 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
                       "Layer");
     
     m_lineHistoryWidgetTabIndex = m_tabWidget->addTab(m_lineHistoryWidget,
-                                                      "Lines");
+                                                      "Dyn-Lines");
     m_metadataWidgetTabIndex = m_tabWidget->addTab(new QWidget(),
                       "Metadata");
     m_tabWidget->setTabEnabled(m_tabWidget->count() - 1, false);
@@ -142,7 +141,6 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     this->setLayoutSpacingAndMargins(layout);
     layout->addWidget(mapNameWidget);
     layout->addWidget(m_tabWidget);
-    //layout->addWidget(windowOptionsWidget);
 
     this->setCentralWidget(w,
                            WuQDialog::SCROLL_AREA_NEVER);
@@ -310,7 +308,7 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
     bool isParcelsValid = false;
     bool isFiberTrajectoryValid = false;
     bool isVolumeLayer = false;
-    bool isLinesValid = false;
+    bool isLineHistoryValid = false;
     
     QString mapFileName = "";
     QString mapName = "";
@@ -407,14 +405,18 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
         else if (m_chartOverlay != NULL) {
             mapFileName = m_caretMappableDataFile->getFileNameNoPath();
 
-            bool hasMapsFlag = false;
-            bool hasHistoryFlag = false;
+            bool hasMapsFlag      = false;
+            bool hasHistoryFlag   = false;
+            bool hasLineLayerFlag = false;
             switch (m_chartOverlay->getChartTwoDataType()) {
                 case ChartTwoDataTypeEnum::CHART_DATA_TYPE_INVALID:
                     break;
                 case ChartTwoDataTypeEnum::CHART_DATA_TYPE_HISTOGRAM:
                     hasMapsFlag = true;
                     break;
+                case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_LAYER:
+                    hasLineLayerFlag = true;
+                    break;
                 case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_SERIES:
                     hasHistoryFlag = true;
                     break;
@@ -497,12 +499,20 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
                 }
             }
             else if (hasHistoryFlag) {
-                isLinesValid = true;
+                isLineHistoryValid = true;
                 m_lineHistoryWidget->updateContent(m_chartOverlay);
                 if (mapName.isEmpty()) {
                     mapName = "Line Chart History";
                 }
             }
+            else if (hasLineLayerFlag) {
+                if (mapName.isEmpty()) {
+                    if ((m_selectedMapFileIndex >= 0)
+                        && (m_selectedMapFileIndex < m_caretMappableDataFile->getNumberOfMaps())) {
+                        mapName = m_caretMappableDataFile->getMapName(m_selectedMapFileIndex);
+                    }
+                }
+            }
         }
         else {
             CaretAssert(0);
@@ -528,7 +538,7 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
     m_tabWidget->setTabEnabled(m_trajectoryWidgetTabIndex,
                                isFiberTrajectoryValid);
     m_tabWidget->setTabEnabled(m_lineHistoryWidgetTabIndex,
-                               isLinesValid);
+                               isLineHistoryValid);
 
     /*
      * When the selected tab is invalid, we want to select the
@@ -588,7 +598,7 @@ OverlaySettingsEditorDialog::updateDialog()
     }
     
     if (m_chartOverlay != NULL) {
-        EventChartOverlayValidate validateChartOverlayEvent(m_chartOverlay);
+        EventChartTwoOverlayValidate validateChartOverlayEvent(m_chartOverlay);
         EventManager::get()->sendEvent(validateChartOverlayEvent.getPointer());
         if ( ! validateChartOverlayEvent.isValidChartOverlay()) {
             m_chartOverlay = NULL;
@@ -611,7 +621,7 @@ void
 OverlaySettingsEditorDialog::updateChartLinesInDialog()
 {
     if (m_chartOverlay != NULL) {
-        EventChartOverlayValidate validateChartOverlayEvent(m_chartOverlay);
+        EventChartTwoOverlayValidate validateChartOverlayEvent(m_chartOverlay);
         EventManager::get()->sendEvent(validateChartOverlayEvent.getPointer());
         if ( ! validateChartOverlayEvent.isValidChartOverlay()) {
             m_chartOverlay = NULL;
