@@ -116,14 +116,16 @@ QGLWidgetTextRenderer::~QGLWidgetTextRenderer()
 void
 QGLWidgetTextRenderer::drawTextAtViewportCoords(const double viewportX,
                                                const double viewportY,
-                                               const AnnotationText& annotationText)
+                                               const AnnotationText& annotationText,
+                                                const BrainOpenGLTextRenderInterface::DrawingFlags& flags)
 {
     setViewportHeight();
     
     drawTextAtViewportCoords(viewportX,
                              viewportY,
                              0.0,
-                             annotationText);
+                             annotationText,
+                             flags);
 }
 
 /**
@@ -145,7 +147,8 @@ void
 QGLWidgetTextRenderer::drawTextAtViewportCoords(const double viewportX,
                                                const double viewportY,
                                                const double /*viewportZ */,
-                                               const AnnotationText& annotationText)
+                                               const AnnotationText& annotationText,
+                                                const BrainOpenGLTextRenderInterface::DrawingFlags& flags)
 {
     setViewportHeight();
     
@@ -193,12 +196,14 @@ QGLWidgetTextRenderer::drawTextAtViewportCoords(const double viewportX,
         case AnnotationTextOrientationEnum::HORIZONTAL:
             drawHorizontalTextAtWindowCoords(viewportX,
                                              viewportY,
-                                             annotationText);
+                                             annotationText,
+                                             flags);
             break;
         case AnnotationTextOrientationEnum::STACKED:
             drawVerticalTextAtWindowCoords(viewportX,
                                            viewportY,
-                                           annotationText);
+                                           annotationText,
+                                           flags);
             break;
     }
     
@@ -211,6 +216,7 @@ QGLWidgetTextRenderer::drawTextAtViewportCoords(const double viewportX,
 /**
  * Draw annnotation text at the given model coordinates using
  * the the annotations attributes for the style of text.
+ * Text is drawn so that is in the plane of the screen (faces user)
  *
  * Depth testing is ENABLED when drawing text with this method.
  *
@@ -222,12 +228,15 @@ QGLWidgetTextRenderer::drawTextAtViewportCoords(const double viewportX,
  *     Model Z-coordinate.
  * @param annotationText
  *     Annotation text and attributes.
+ * @param flags
+ *     Drawing flags.
  */
 void
-QGLWidgetTextRenderer::drawTextAtModelCoords(const double modelX,
+QGLWidgetTextRenderer::drawTextAtModelCoordsFacingUser(const double modelX,
                                           const double modelY,
                                           const double modelZ,
-                                          const AnnotationText& annotationText)
+                                          const AnnotationText& annotationText,
+                                             const BrainOpenGLTextRenderInterface::DrawingFlags& flags)
 {
     setViewportHeight();
     
@@ -289,8 +298,8 @@ QGLWidgetTextRenderer::drawTextAtModelCoords(const double modelX,
         drawTextAtViewportCoords(x,
                                  y,
                                  0.0,
-                                 annotationText);
-        
+                                 annotationText,
+                                 flags);
 //        glPopMatrix();
 //        glMatrixMode(GL_PROJECTION);
 //        glPopMatrix();
@@ -301,6 +310,32 @@ QGLWidgetTextRenderer::drawTextAtModelCoords(const double modelX,
     }
 }
 
+/**
+ * Draw text in model space using the current model transformations.
+ *
+ * Depth testing is ENABLED when drawing text with this method.
+ *
+ * @param annotationText
+ *     Annotation text and attributes.
+ * @param modelSpaceScaling
+ *   Scaling in the model space.
+ * @param heightOrWidthForPercentageSizeText
+ *    If positive, use it to override width/height of viewport.
+ * @param normalVector
+ *     Normal vector of text.
+ * @param flags
+ *     Drawing flags.
+ */
+void
+QGLWidgetTextRenderer::drawTextInModelSpace(const AnnotationText& /* annotationText */,
+                                            const float /*modelSpaceScaling*/,
+                                            const float /*heightOrWidthForPercentageSizeText*/,
+                                            const float* /*normalVector[3]*/,
+                                            const DrawingFlags& /* flags */)
+{
+    CaretAssertMessage(0, "Not implemented");
+}
+
 /**
  * Draw horizontal annotation text at the given window coordinates.
  *
@@ -317,7 +352,8 @@ QGLWidgetTextRenderer::drawTextAtModelCoords(const double modelX,
 void
 QGLWidgetTextRenderer::drawHorizontalTextAtWindowCoords(const double windowX,
                                                        const double windowY,
-                                                       const AnnotationText& annotationText)
+                                                       const AnnotationText& annotationText,
+                                                        const BrainOpenGLTextRenderInterface::DrawingFlags& flags)
 {
     QFont* font = findFont(annotationText,
                            false);
@@ -353,7 +389,7 @@ QGLWidgetTextRenderer::drawHorizontalTextAtWindowCoords(const double windowX,
     const AString text = annotationText.getText();
     
     double bottomLeft[3], bottomRight[3], topRight[3], topLeft[3];
-    getBoundsForTextAtViewportCoords(annotationText, windowX, windowY, 0.0, viewport[2], viewport[3], bottomLeft, bottomRight, topRight, topLeft);
+    getBoundsForTextAtViewportCoords(annotationText, flags, windowX, windowY, 0.0, viewport[2], viewport[3], bottomLeft, bottomRight, topRight, topLeft);
     
     double left   = bottomLeft[0];
     double right  = bottomRight[0];
@@ -498,7 +534,8 @@ QGLWidgetTextRenderer::drawHorizontalTextAtWindowCoords(const double windowX,
 void
 QGLWidgetTextRenderer::drawVerticalTextAtWindowCoords(const double windowX,
                                                      const double windowY,
-                                                     const AnnotationText& annotationText)
+                                                     const AnnotationText& annotationText,
+                                                      const BrainOpenGLTextRenderInterface::DrawingFlags& flags)
 {
     QFont* font = findFont(annotationText,
                            false);
@@ -538,6 +575,7 @@ QGLWidgetTextRenderer::drawVerticalTextAtWindowCoords(const double windowX,
     double textHeight = 0.0;
     std::vector<CharInfo> textCharsToDraw;
     getVerticalTextCharInfo(annotationText,
+                            flags,
                             textMinX,
                             textMaxX,
                             textHeight,
@@ -696,6 +734,7 @@ QGLWidgetTextRenderer::applyForegroundColoring(const AnnotationText& annotationT
  */
 void
 QGLWidgetTextRenderer::getTextWidthHeightInPixels(const AnnotationText& annotationText,
+                                                  const BrainOpenGLTextRenderInterface::DrawingFlags& flags,
                                                   const double viewportWidth,
                                                   const double viewportHeight,
                                                  double& widthOut,
@@ -710,6 +749,7 @@ QGLWidgetTextRenderer::getTextWidthHeightInPixels(const AnnotationText& annotati
     
     double bottomLeft[3], bottomRight[3], topRight[3], topLeft[3];
     getBoundsForTextAtViewportCoords(annotationText,
+                                     flags,
                                      0.0,
                                      0.0,
                                      0.0,
@@ -724,6 +764,45 @@ QGLWidgetTextRenderer::getTextWidthHeightInPixels(const AnnotationText& annotati
     heightOut = yMax - yMin;
 }
 
+/**
+ * Get the bounds of text drawn in model space using the current model transformations.
+ *
+ * @param annotationText
+ *   Text that is to be drawn.
+ * @param modelSpaceScaling
+ *   Scaling in the model space.
+ * @param heightOrWidthForPercentageSizeText
+ *    Size of region used when converting percentage size to a fixed size
+ * @param flags
+ *     Drawing flags.
+ * @param bottomLeftOut
+ *    The bottom left corner of the text bounds.
+ * @param bottomRightOut
+ *    The bottom right corner of the text bounds.
+ * @param topRightOut
+ *    The top right corner of the text bounds.
+ * @param topLeftOut
+ *    The top left corner of the text bounds.
+ * @param underlineStartOut
+ *    Starting coordinate for drawing text underline.
+ * @param underlineEndOut
+ *    Ending coordinate for drawing text underline.
+ */
+void
+QGLWidgetTextRenderer::getBoundsForTextInModelSpace(const AnnotationText& /*annotationText*/,
+                                                    const float /*modelSpaceScaling*/,
+                                                    const float /*heightOrWidthForPercentageSizeText*/,
+                                                    const DrawingFlags& /*flags*/,
+                                                    double* /*bottomLeftOut[3]*/,
+                                                    double* /*bottomRightOut[3]*/,
+                                                    double* /*topRightOut[3]*/,
+                                                    double* /*topLeftOut[3]*/,
+                                                    double* /*underlineStartOut[3]*/,
+                                                    double* /*underlineEndOut[3]*/)
+{
+    
+}
+
 /**
  * Get the bounds of text (in pixels) using the given text
  * attributes.
@@ -753,6 +832,7 @@ QGLWidgetTextRenderer::getTextWidthHeightInPixels(const AnnotationText& annotati
  */
 void
 QGLWidgetTextRenderer::getBoundsForTextAtViewportCoords(const AnnotationText& annotationText,
+                                                        const BrainOpenGLTextRenderInterface::DrawingFlags& flags,
                                               const double viewportX,
                                               const double viewportY,
                                               const double viewportZ,
@@ -797,6 +877,7 @@ QGLWidgetTextRenderer::getBoundsForTextAtViewportCoords(const AnnotationText& an
             double textHeight = 0.0;
             std::vector<CharInfo> charInfo;
             getVerticalTextCharInfo(annotationText,
+                                    flags,
                                     xMin,
                                     xMax,
                                     textHeight,
@@ -853,6 +934,137 @@ QGLWidgetTextRenderer::getBoundsForTextAtViewportCoords(const AnnotationText& an
     topLeftOut[0] = viewportZ;
 }
 
+/**
+ * Get the bounds of text (in pixels) using the given text
+ * attributes.  NO MARGIN is placed around the text.
+ *
+ * See http://ftgl.sourceforge.net/docs/html/metrics.png
+ *
+ * @param annotationText
+ *   Text that is to be drawn.
+ * @param viewportX
+ *    Viewport X-coordinate.
+ * @param viewportY
+ *    Viewport Y-coordinate.
+ * @param viewportZ
+ *    Viewport Z-coordinate.
+ * @param viewportWidth
+ *    Height of the viewport needed for percentage height text.
+ * @param viewportHeight
+ *    Height of the viewport needed for percentage height text.
+ * @param bottomLeftOut
+ *    The bottom left corner of the text bounds.
+ * @param bottomRightOut
+ *    The bottom right corner of the text bounds.
+ * @param topRightOut
+ *    The top right corner of the text bounds.
+ * @param topLeftOut
+ *    The top left corner of the text bounds.
+ */
+void
+QGLWidgetTextRenderer::getBoundsWithoutMarginForTextAtViewportCoords(const AnnotationText& annotationText,
+                                                                     const BrainOpenGLTextRenderInterface::DrawingFlags& flags,
+                                                                     const double viewportX,
+                                                                     const double viewportY,
+                                                                     const double viewportZ,
+                                                                     const double /*viewportWidth*/,
+                                                                     const double /*viewportHeight*/,
+                                                                     double bottomLeftOut[3],
+                                                                     double bottomRightOut[3],
+                                                                     double topRightOut[3],
+                                                                     double topLeftOut[3])
+{
+    setViewportHeight();
+    
+    QFont* font = findFont(annotationText,
+                           false);
+    if (font == NULL) {
+        return;
+    }
+    
+    double xMin = 0.0;
+    double xMax = 0.0;
+    double yMin = 0.0;
+    double yMax = 0.0;
+    
+    switch (annotationText.getOrientation()) {
+        case AnnotationTextOrientationEnum::HORIZONTAL:
+        {
+            QFontMetricsF fontMetrics(*font);
+            QRectF boundsRect = fontMetrics.boundingRect(annotationText.getText());
+            
+            /*
+             * Note: sometimes boundsRect.top() is negative and
+             * that screws things up.
+             */
+            xMin = boundsRect.left();
+            xMax = boundsRect.right();
+            yMin = boundsRect.bottom();
+            yMax = boundsRect.bottom() + boundsRect.height();
+        }
+            break;
+        case AnnotationTextOrientationEnum::STACKED:
+        {
+            double textHeight = 0.0;
+            std::vector<CharInfo> charInfo;
+            getVerticalTextCharInfo(annotationText,
+                                    flags,
+                                    xMin,
+                                    xMax,
+                                    textHeight,
+                                    charInfo);
+            yMax = textHeight;
+        }
+            break;
+    }
+    
+    const double width = xMax - xMin;
+    double left = 0.0;
+    switch (annotationText.getHorizontalAlignment()) {
+        case AnnotationTextAlignHorizontalEnum::LEFT:
+            left = viewportX;
+            break;
+        case AnnotationTextAlignHorizontalEnum::CENTER:
+            left = viewportX - (width / 2.0);
+            break;
+        case AnnotationTextAlignHorizontalEnum::RIGHT:
+            left = viewportX - width;
+            break;
+    }
+    const double right = left + width;
+    
+    const double height = yMax - yMin;
+    double bottom = 0.0;
+    switch (annotationText.getVerticalAlignment()) {
+        case AnnotationTextAlignVerticalEnum::BOTTOM:
+            bottom = viewportY;
+            break;
+        case AnnotationTextAlignVerticalEnum::MIDDLE:
+            bottom = viewportY - (height / 2.0);
+            break;
+        case AnnotationTextAlignVerticalEnum::TOP:
+            bottom = viewportY - height;
+            break;
+    }
+    const double top = bottom + height;
+    
+    bottomLeftOut[0] = left;
+    bottomLeftOut[1] = bottom;
+    bottomLeftOut[2] = viewportZ;
+    
+    bottomRightOut[0] = right;
+    bottomRightOut[0] = bottom;
+    bottomRightOut[0] = viewportZ;
+    
+    topRightOut[0] = right;
+    topRightOut[0] = top;
+    topRightOut[0] = viewportZ;
+    
+    topLeftOut[0] = left;
+    topLeftOut[0] = top;
+    topLeftOut[0] = viewportZ;
+}
+
 /**
  * Get the character info for drawing vertical text which includes
  * position for each of the characters.  The TOP of the first
@@ -872,6 +1084,7 @@ QGLWidgetTextRenderer::getBoundsForTextAtViewportCoords(const AnnotationText& an
  */
 void
 QGLWidgetTextRenderer::getVerticalTextCharInfo(const AnnotationText& annotationText,
+                                               const BrainOpenGLTextRenderInterface::DrawingFlags& flags,
                                               double& xMinOut,
                                               double& xMaxOut,
                                               double& heightOut,
@@ -882,7 +1095,9 @@ QGLWidgetTextRenderer::getVerticalTextCharInfo(const AnnotationText& annotationT
     xMaxOut = 0.0;
     heightOut = 0.0;
     
-    const AString text = annotationText.getText();
+    const AString text = (flags.isDrawSubstitutedText()
+                          ? annotationText.getTextWithSubstitutionsApplied()
+                          : annotationText.getText());
     const int32_t numChars = static_cast<int32_t>(text.size());
     if (numChars <= 0) {
         return;
